@@ -240,6 +240,24 @@ static int connection_handle_read(server *srv, connection *con) {
 			case SSL_ERROR_WANT_READ:
 				return 0;
 			case SSL_ERROR_SYSCALL:
+				/**
+				 * man SSL_get_error()
+				 * 
+				 * SSL_ERROR_SYSCALL
+				 *   Some I/O error occurred.  The OpenSSL error queue may contain more 
+				 *   information on the error.  If the error queue is empty (i.e.
+				 *   ERR_get_error() returns 0), ret can be used to find out more about 
+				 *   the error: If ret == 0, an EOF was observed that violates the
+				 *   protocol.  If ret == -1, the underlying BIO reported an I/O error 
+				 *   (for socket I/O on Unix systems, consult errno for details).
+				 *
+				 */
+				while((ssl_err = ERR_get_error())) {
+					/* get all errors from the error-queue */
+					log_error_write(srv, __FILE__, __LINE__, "sds", "SSL:", 
+							r, ERR_error_string(ssl_err, NULL));
+				}
+
 				switch(errno) {
 				default:
 					log_error_write(srv, __FILE__, __LINE__, "sddds", "SSL:", 
@@ -258,14 +276,10 @@ static int connection_handle_read(server *srv, connection *con) {
 				
 				/* fall thourgh */
 			default:
-				ssl_err = ERR_get_error();
-				switch(ssl_err) {
-				case SSL_F_SSL23_GET_CLIENT_HELLO:
-					/* a unencrypted HTTP request on a HTTPS socket. Do a redirect to the right location */
-				default:
+				while((ssl_err = ERR_get_error())) {
+					/* get all errors from the error-queue */
 					log_error_write(srv, __FILE__, __LINE__, "sds", "SSL:", 
 							r, ERR_error_string(ssl_err, NULL));
-					break;
 				}
 				break;
 			}
@@ -301,12 +315,10 @@ static int connection_handle_read(server *srv, connection *con) {
 	} else if (len == 0) {
 		con->is_readable = 0;
 		/* the other end close the connection -> KEEP-ALIVE */
-#if 0
-		log_error_write(srv, __FILE__, __LINE__, "s",
-				"connection closed: remote site closed unexpectedly");
-#endif 
-		connection_set_state(srv, con, CON_STATE_ERROR);
-		return -1;
+
+		/* pipelining */
+
+		return -2;
 	} else if ((size_t)len < b->size - 1) {
 		/* we got less then expected, wait for the next fd-event */
 		
@@ -340,7 +352,13 @@ static int connection_handle_write_prepare(server *srv, connection *con) {
 		case HTTP_METHOD_PROPPATCH:
 			break;
 		case HTTP_METHOD_OPTIONS:
-			if (con->uri.path->ptr[0] != '*') {
+			/*
+			 * 400 is coming from the request-parser BEFORE uri.path is set
+			 * 403 is from the response handler when noone else catched it 
+			 * 
+			 * */
+			if (con->uri.path->used &&
+			    con->uri.path->ptr[0] != '*') {
 				response_header_insert(srv, con, CONST_STR_LEN("Allow"), CONST_STR_LEN("OPTIONS, GET, HEAD, POST"));
 
 				con->http_status = 200;
@@ -352,6 +370,7 @@ static int connection_handle_write_prepare(server *srv, connection *con) {
 		default:
 			switch(con->http_status) {
 			case 400: /* bad request */
+			case 414: /* overload request header */
 			case 505: /* unknown protocol */
 			case 207: /* this was webdav */
 				break;
@@ -396,6 +415,7 @@ static int connection_handle_write_prepare(server *srv, connection *con) {
 				con->file_finished = 1;
 				
 				http_chunk_append_file(srv, con, con->physical.path, 0, sce->st.st_size);
+				response_header_overwrite(srv, con, CONST_STR_LEN("Content-Type"), CONST_BUF_LEN(sce->content_type));
 			}
 		}
 		
@@ -438,6 +458,7 @@ static int connection_handle_write_prepare(server *srv, connection *con) {
 		/* fall through */
 	case 207:
 	case 200: /* class: header + body */
+	case 301:
 	case 302:
 		break;
 		
@@ -446,7 +467,6 @@ static int connection_handle_write_prepare(server *srv, connection *con) {
 		
 		break;
 	case 205: /* class: header only */
-	case 301:
 	case 304:
 	default:
 		/* disable chunked encoding again as we have no body */
@@ -461,7 +481,8 @@ static int connection_handle_write_prepare(server *srv, connection *con) {
 	if (con->file_finished) {
 		/* we have all the content and chunked encoding is not used, set a content-length */ 
 		
-		if ((con->response.transfer_encoding & HTTP_TRANSFER_ENCODING_CHUNKED) == 0) {
+		if ((!(con->parsed_response & HTTP_CONTENT_LENGTH)) && 
+		    (con->response.transfer_encoding & HTTP_TRANSFER_ENCODING_CHUNKED) == 0) {
 			buffer_copy_off_t(srv->tmp_buf, chunkqueue_length(con->write_queue));
 		
 			response_header_overwrite(srv, con, CONST_STR_LEN("Content-Length"), CONST_BUF_LEN(srv->tmp_buf));
@@ -580,13 +601,15 @@ connection *connection_init(server *srv) {
 	con->write_queue = chunkqueue_init();
 	con->read_queue = chunkqueue_init();
 	con->request_content_queue = chunkqueue_init();
+	chunkqueue_set_tempdirs(con->request_content_queue, srv->srvconf.upload_tempdirs);
+
 	con->request.headers      = array_init();
 	con->response.headers     = array_init();
 	con->environment     = array_init();
 	
 	/* init plugin specific connection structures */
 	
-	con->plugin_ctx = calloc(srv->plugins.used + 1, sizeof(void *));
+	con->plugin_ctx = calloc(1, (srv->plugins.used + 1) * sizeof(void *));
 	
 	con->cond_cache = calloc(srv->config_context->used, sizeof(cond_cache_t));
 	config_setup_connection(srv, con);
@@ -735,6 +758,8 @@ int connection_reset(server *srv, connection *con) {
 		plugin *p = ((plugin **)(srv->plugins.ptr))[i];
 		plugin_data *pd = p->data;
 
+		if (!pd) continue;
+
 		if (con->plugin_ctx[pd->id] != NULL) {
 			log_error_write(srv, __FILE__, __LINE__, "sb", "missing cleanup in", p->name);
 		}
@@ -802,37 +827,73 @@ char *buffer_search_rnrn(buffer *b) {
 	
 	return NULL;
 }
-
+/**
+ * handle all header and content read
+ *
+ * we get called by the state-engine and by the fdevent-handler
+ */
 int connection_handle_read_state(server *srv, connection *con)  {
 	int ostate = con->state;
 	char *h_term = NULL;
 	chunk *c;
 	chunkqueue *cq = con->read_queue;
 	chunkqueue *dst_cq = con->request_content_queue;
-	size_t memusage;
 	
 	if (con->is_readable) {
 		con->read_idle_ts = srv->cur_ts;
 	
-		if (0 != connection_handle_read(srv, con)) {
+		switch(connection_handle_read(srv, con)) {
+		case -1:
 			return -1;
+		case -2:
+			/* remote side closed the connection
+			 * if we still have content, handle it, if not leave here */
+
+			if (cq->first == cq->last &&
+			    cq->first->mem->used == 0) {
+
+				/* conn-closed, leave here */
+				connection_set_state(srv, con, CON_STATE_ERROR);
+			}
+		default:
+			break;
 		}
 	}
 
-	/* move the empty chunks out of the way */
-	for (c = cq->first; c; c = cq->first) {
-		assert(c != c->next);
-		
-		if (c->mem->used == 0) {
+	/* the last chunk might be empty */
+	for (c = cq->first; c;) {
+		if (cq->first == c && c->mem->used == 0) {
+			/* the first node is empty */
+			/* ... and it is empty, move it to unused */
+
 			cq->first = c->next;
+			if (cq->first == NULL) cq->last = NULL;
+
 			c->next = cq->unused;
 			cq->unused = c;
-			
-			if (cq->first == NULL) cq->last = NULL;
-			
+			cq->unused_chunks++;
+
 			c = cq->first;
+		} else if (c->next && c->next->mem->used == 0) {
+			chunk *fc;
+			/* next node is the last one */
+			/* ... and it is empty, move it to unused */
+
+			fc = c->next;
+			c->next = fc->next;
+
+			fc->next = cq->unused;
+			cq->unused = fc;
+			cq->unused_chunks++;
+
+			/* the last node was empty */
+			if (c->next == NULL) {
+				cq->last = c;
+			} 
+
+			c = c->next;
 		} else {
-			break;
+			c = c->next;
 		}
 	}
 	
@@ -929,14 +990,16 @@ int connection_handle_read_state(server *srv, connection *con)  {
 		if (h_term) {
 			connection_set_state(srv, con, CON_STATE_REQUEST_END);
 		} else if (con->request.request->used > 64 * 1024) {
-			log_error_write(srv, __FILE__, __LINE__, "sd", "http-header larger then 64k -> disconnected", chunkqueue_length(cq));
-			connection_set_state(srv, con, CON_STATE_ERROR);
+			log_error_write(srv, __FILE__, __LINE__, "s", "oversized request-header -> sending Status 414");
+
+			con->http_status = 414; /* Request-URI too large */
+			con->keep_alive = 0;
+			connection_set_state(srv, con, CON_STATE_HANDLE_REQUEST);
 		}
 		break;
 	case CON_STATE_READ_POST: 
-		for (c = cq->first; c && (dst_cq->bytes_in != con->request.content_length); c = c->next) {
+		for (c = cq->first; c && (dst_cq->bytes_in != (off_t)con->request.content_length); c = c->next) {
 			off_t weWant, weHave, toRead;
-			int buffer_to_file = 0;
 			
 			weWant = con->request.content_length - dst_cq->bytes_in;
 			
@@ -964,25 +1027,73 @@ int connection_handle_read_state(server *srv, connection *con)  {
 				if (dst_cq->last &&
 				    dst_cq->last->type == FILE_CHUNK &&
 				    dst_cq->last->file.is_temp &&
-				    dst_cq->last->offset == 0 &&
-				    dst_cq->last->file.length < 1 * 1024 * 1024) {
+				    dst_cq->last->offset == 0) {
 					/* ok, take the last chunk for our job */
-					dst_c = dst_cq->last;
-					dst_c->file.fd = open(dst_c->file.name->ptr, O_WRONLY | O_APPEND);
+
+			 		if (dst_cq->last->file.length < 1 * 1024 * 1024) {
+						dst_c = dst_cq->last;
+
+						if (dst_c->file.fd == -1) {
+							/* this should not happen as we cache the fd, but you never know */
+							dst_c->file.fd = open(dst_c->file.name->ptr, O_WRONLY | O_APPEND);
+						}
+					} else {
+						/* the chunk is too large now, close it */
+						dst_c = dst_cq->last;
+
+						if (dst_c->file.fd != -1) {
+							close(dst_c->file.fd);
+							dst_c->file.fd = -1;
+						}
+						dst_c = chunkqueue_get_append_tempfile(dst_cq);
+					}
 				} else {
 					dst_c = chunkqueue_get_append_tempfile(dst_cq);
 				}
 
 				/* we have a chunk, let's write to it */
 
-				assert(dst_c->file.fd != -1);
+				if (dst_c->file.fd == -1) {
+					/* we don't have file to write to, 
+					 * EACCES might be one reason.
+					 *
+					 * Instead of sending 500 we send 413 and say the request is too large
+					 *  */
 
-				assert(toRead == write(dst_c->file.fd, c->mem->ptr + c->offset, toRead));
+					log_error_write(srv, __FILE__, __LINE__, "sbs",
+							"denying upload as opening to temp-file for upload failed:", 
+							dst_c->file.name, strerror(errno));
+
+					con->http_status = 413; /* Request-Entity too large */
+					con->keep_alive = 0;
+					connection_set_state(srv, con, CON_STATE_HANDLE_REQUEST);
+
+					break;
+				}
+
+				if (toRead != write(dst_c->file.fd, c->mem->ptr + c->offset, toRead)) {
+					/* write failed for some reason ... disk full ? */ 
+					log_error_write(srv, __FILE__, __LINE__, "sbs",
+							"denying upload as writing to file failed:", 
+							dst_c->file.name, strerror(errno));
+					
+					con->http_status = 413; /* Request-Entity too large */
+					con->keep_alive = 0;
+					connection_set_state(srv, con, CON_STATE_HANDLE_REQUEST);
+				
+					close(dst_c->file.fd);
+					dst_c->file.fd = -1;
+
+					break;
+				}
 
 				dst_c->file.length += toRead;
 					
-				close(dst_c->file.fd);
-				dst_c->file.fd = -1;
+				if (dst_cq->bytes_in + toRead == (off_t)con->request.content_length) {
+					/* we read everything, close the chunk */
+					close(dst_c->file.fd);
+					dst_c->file.fd = -1;
+				}
 			} else {
 				buffer *b;
 
@@ -995,7 +1106,7 @@ int connection_handle_read_state(server *srv, connection *con)  {
 		}
 
 		/* Content is ready */
-		if (dst_cq->bytes_in == con->request.content_length) {
+		if (dst_cq->bytes_in == (off_t)con->request.content_length) {
 			connection_set_state(srv, con, CON_STATE_HANDLE_REQUEST);
 		}
 			
@@ -1278,8 +1389,12 @@ int connection_state_machine(server *srv, connection *con) {
 						/* error-handler is a 404 */
 						
 						/* continue as normal, status is the same */
-						log_error_write(srv, __FILE__, __LINE__, "sb", "error-handler not found:", con->conf.error_handler);
-						
+						log_error_write(srv, __FILE__, __LINE__, "sb", 
+								"Warning: Either the error-handler returned status 404 or the error-handler itself was not found:", con->request.uri);
+						log_error_write(srv, __FILE__, __LINE__, "sd", 
+								"returning the original status", con->error_handler_saved_status);
+						log_error_write(srv, __FILE__, __LINE__, "s", 
+								"If this is a rails app: check your production.log");
 						con->http_status = con->error_handler_saved_status;
 					}
 				} else if (con->in_error_handler) {
