@@ -5,6 +5,16 @@
 #include "array.h"
 #include "fastcgi.h"
 
+static data_unset *data_fastcgi_copy(const data_unset *s) {
+	data_fastcgi *src = (data_fastcgi *)s;
+	data_fastcgi *ds = data_fastcgi_init();
+
+	buffer_copy_string_buffer(ds->key, src->key);
+	buffer_copy_string_buffer(ds->host, src->host);
+	ds->is_index_key = src->is_index_key;
+	return (data_unset *)ds;
+}
+
 static void data_fastcgi_free(data_unset *d) {
 	data_fastcgi *ds = (data_fastcgi *)d;
 	
@@ -30,10 +40,11 @@ static int data_fastcgi_insert_dup(data_unset *dst, data_unset *src) {
 	return 0;
 }
 
-static void data_fastcgi_print(data_unset *d) {
+static void data_fastcgi_print(const data_unset *d, int depth) {
 	data_fastcgi *ds = (data_fastcgi *)d;
+	UNUSED(depth);
 	
-	printf("{%s: %s}", ds->key->ptr, ds->host->ptr);
+	fprintf(stderr, "fastcgi(%s)", ds->host->ptr);
 }
 
 
@@ -47,6 +58,7 @@ data_fastcgi *data_fastcgi_init(void) {
 	ds->port = 0;
 	ds->is_disabled = 0;
 	
+	ds->copy = data_fastcgi_copy;
 	ds->free = data_fastcgi_free;
 	ds->reset = data_fastcgi_reset;
 	ds->insert_dup = data_fastcgi_insert_dup;
