@@ -27,7 +27,8 @@ typedef enum {
 	HTTP_METHOD_CHECKIN,
 	HTTP_METHOD_VERSION_CONTROL,
 	HTTP_METHOD_UNCHECKOUT,
-	HTTP_METHOD_LABEL
+	HTTP_METHOD_LABEL,
+	HTTP_METHOD_CONNECT
 } http_method_t;
 
 typedef enum { HTTP_VERSION_UNSET = -1, HTTP_VERSION_1_0, HTTP_VERSION_1_1 } http_version_t;
