@@ -12,6 +12,8 @@
 #include "log.h"
 #include "response.h"
 
+handler_t auth_ldap_init(server *srv, mod_auth_plugin_config *s);
+
 
 /**
  * the basic and digest auth framework
@@ -191,11 +193,23 @@ static handler_t mod_auth_uri_handler(server *srv, connection *con, void *p_d) {
 	
 	/* search auth-directives for path */
 	for (k = 0; k < p->conf.auth_require->used; k++) {
-		if (p->conf.auth_require->data[k]->key->used == 0) continue;
-		
-		if (0 == strncmp(con->uri.path->ptr, p->conf.auth_require->data[k]->key->ptr, p->conf.auth_require->data[k]->key->used - 1)) {
-			auth_required = 1;
-			break;
+		buffer *req = p->conf.auth_require->data[k]->key;
+
+		if (req->used == 0) continue;
+		if (con->uri.path->used < req->used) continue;
+
+		/* if we have a case-insensitive FS we have to lower-case the URI here too */
+
+		if (con->conf.force_lowercase_filenames) {
+			if (0 == strncasecmp(con->uri.path->ptr, req->ptr, req->used - 1)) {
+				auth_required = 1;
+				break;
+			}
+		} else {
+			if (0 == strncmp(con->uri.path->ptr, req->ptr, req->used - 1)) {
+				auth_required = 1;
+				break;
+			}
 		}
 	}
 	
@@ -505,8 +519,23 @@ SETDEFAULTS_FUNC(mod_auth_set_defaults) {
 				close(fd);
 			}
 			break;
-		case AUTH_BACKEND_LDAP: 
+		case AUTH_BACKEND_LDAP: {
+			handler_t ret = auth_ldap_init(srv, s);
+			if (ret == HANDLER_ERROR)
+				return (ret);
+                        break;
+		}
+                default:
+                        break;
+                }
+        }
+
+        return HANDLER_GO_ON;
+}
+
+handler_t auth_ldap_init(server *srv, mod_auth_plugin_config *s) {
 #ifdef USE_LDAP
+			int ret;
 #if 0			
 			if (s->auth_ldap_basedn->used == 0) {
 				log_error_write(srv, __FILE__, __LINE__, "s", "ldap: auth.backend.ldap.base-dn has to be set");
@@ -531,7 +560,6 @@ SETDEFAULTS_FUNC(mod_auth_set_defaults) {
 			}
 			
 			if (s->auth_ldap_hostname->used) {
-				int ret;
 				if (NULL == (s->ldap = ldap_init(s->auth_ldap_hostname->ptr, LDAP_PORT))) {
 					log_error_write(srv, __FILE__, __LINE__, "ss", "ldap ...", strerror(errno));
 					
@@ -546,17 +574,16 @@ SETDEFAULTS_FUNC(mod_auth_set_defaults) {
 				}
 
 				if (s->auth_ldap_starttls) {
-					if (buffer_is_empty(s->auth_ldap_cafile)) {
-						log_error_write(srv, __FILE__, __LINE__, "s", "CA file has to be set");
+					/* if no CA file is given, it is ok, as we will use encryption 
+					 * if the server requires a CAfile it will tell us */
+					if (!buffer_is_empty(s->auth_ldap_cafile)) {
+						if (LDAP_OPT_SUCCESS != (ret = ldap_set_option(NULL, LDAP_OPT_X_TLS_CACERTFILE, 
+										s->auth_ldap_cafile->ptr))) {
+							log_error_write(srv, __FILE__, __LINE__, "ss", 
+									"Loading CA certificate failed:", ldap_err2string(ret));
 						
-						return HANDLER_ERROR;
-						
-					}
-			
-					if (LDAP_OPT_SUCCESS != (ret = ldap_set_option(NULL, LDAP_OPT_X_TLS_CACERTFILE, s->auth_ldap_cafile->ptr))) {
-						log_error_write(srv, __FILE__, __LINE__, "ss", "Loading CA certificate failed:", ldap_err2string(ret));
-						
-						return HANDLER_ERROR;
+							return HANDLER_ERROR;
+						}
 					}
 	
 					if (LDAP_OPT_SUCCESS != (ret = ldap_start_tls_s(s->ldap, NULL,  NULL))) {
@@ -586,13 +613,7 @@ SETDEFAULTS_FUNC(mod_auth_set_defaults) {
 			log_error_write(srv, __FILE__, __LINE__, "s", "no ldap support available");
 			return HANDLER_ERROR;
 #endif
-			break;
-		default:
-			break;
-		}
-	}
-	
-	return HANDLER_GO_ON;
+		return HANDLER_GO_ON;
 }
 
 int mod_auth_plugin_init(plugin *p) {
