@@ -25,7 +25,6 @@ INIT_FUNC(mod_cml_init) {
 	
 	p->basedir         = buffer_init();
 	p->baseurl         = buffer_init();
-	p->session_id      = buffer_init();
 	p->trigger_handler = buffer_init();
 	
 	return p;
@@ -47,6 +46,7 @@ FREE_FUNC(mod_cml_free) {
 			buffer_free(s->ext);
 			
 			buffer_free(s->mc_namespace);
+			buffer_free(s->power_magnet);
 			array_free(s->mc_hosts);
 			
 #if defined(HAVE_MEMCACHE_H)
@@ -59,7 +59,6 @@ FREE_FUNC(mod_cml_free) {
 	}
 	
 	buffer_free(p->trigger_handler);
-	buffer_free(p->session_id);
 	buffer_free(p->basedir);
 	buffer_free(p->baseurl);
 	
@@ -78,6 +77,7 @@ SETDEFAULTS_FUNC(mod_cml_set_defaults) {
 		{ "cml.extension",              NULL, T_CONFIG_STRING, T_CONFIG_SCOPE_CONNECTION },       /* 0 */
 		{ "cml.memcache-hosts",         NULL, T_CONFIG_ARRAY, T_CONFIG_SCOPE_CONNECTION },        /* 1 */
 		{ "cml.memcache-namespace",     NULL, T_CONFIG_STRING, T_CONFIG_SCOPE_CONNECTION },       /* 2 */
+		{ "cml.power-magnet",           NULL, T_CONFIG_STRING, T_CONFIG_SCOPE_CONNECTION },       /* 3 */
 		{ NULL,                         NULL, T_CONFIG_UNSET, T_CONFIG_SCOPE_UNSET }
 	};
 	
@@ -92,6 +92,7 @@ SETDEFAULTS_FUNC(mod_cml_set_defaults) {
 		s->ext    = buffer_init();
 		s->mc_hosts       = array_init();
 		s->mc_namespace   = buffer_init();
+		s->power_magnet   = buffer_init();
 #if defined(HAVE_MEMCACHE_H)
 		s->mc = NULL;
 #endif
@@ -99,6 +100,7 @@ SETDEFAULTS_FUNC(mod_cml_set_defaults) {
 		cv[0].destination = s->ext;
 		cv[1].destination = s->mc_hosts;
 		cv[2].destination = s->mc_namespace;
+		cv[3].destination = s->power_magnet;
 		
 		p->config_storage[i] = s;
 	
@@ -144,6 +146,7 @@ static int mod_cml_patch_connection(server *srv, connection *con, plugin_data *p
 	PATCH(mc);
 #endif
 	PATCH(mc_namespace);
+	PATCH(power_magnet);
 	
 	/* skip the first, the global context */
 	for (i = 1; i < srv->config_context->used; i++) {
@@ -165,6 +168,8 @@ static int mod_cml_patch_connection(server *srv, connection *con, plugin_data *p
 #endif
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("cml.memcache-namespace"))) {
 				PATCH(mc_namespace);
+			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("cml.power-magnet"))) {
+				PATCH(power_magnet);
 			}
 		}
 	}
@@ -173,159 +178,11 @@ static int mod_cml_patch_connection(server *srv, connection *con, plugin_data *p
 }
 #undef PATCH
 
-
-int cache_get_cookie_session_id(server *srv, connection *con, plugin_data *p) {
-	data_unset *d;
-	
-	if (NULL != (d = array_get_element(con->request.headers, "Cookie"))) {
-		data_string *ds = (data_string *)d;
-		size_t key = 0, value = 0;
-		size_t is_key = 1, is_sid = 0;
-		size_t i;
-		
-		/* found COOKIE */
-		if (!DATA_IS_STRING(d)) return -1;
-		if (ds->value->used == 0) return -1;
-			
-		if (ds->value->ptr[0] == '\0' ||
-		    ds->value->ptr[0] == '=' ||
-		    ds->value->ptr[0] == ';') return -1;
-		
-		buffer_reset(p->session_id);
-		for (i = 0; i < ds->value->used; i++) {
-			switch(ds->value->ptr[i]) {
-			case '=':
-				if (is_key) {
-					if (0 == strncmp(ds->value->ptr + key, "PHPSESSID", i - key)) {
-						/* found PHP-session-id-key */
-						is_sid = 1;
-					}
-					value = i + 1;
-				
-					is_key = 0;
-				}
-				
-				break;
-			case ';':
-				if (is_sid) {
-					buffer_copy_string_len(p->session_id, ds->value->ptr + value, i - value);
-				}
-				
-				is_sid = 0;
-				key = i + 1;
-				value = 0;
-				is_key = 1;
-				break;
-			case ' ':
-				if (is_key == 1 && key == i) key = i + 1;
-				if (is_key == 0 && value == i) value = i + 1;
-				break;
-			case '\0':
-				if (is_sid) {
-					buffer_copy_string_len(p->session_id, ds->value->ptr + value, i - value);
-				}
-				/* fin */
-				break;
-			}
-		}
-		
-		if (!buffer_is_empty(p->session_id)) {
-			log_error_write(srv, __FILE__, __LINE__, "sb", 
-				"Session-ID", p->session_id);
-		}
-	}
-	
-	return !buffer_is_empty(p->session_id);
-}
-
-int cache_get_url_session_id(server *srv, connection *con, plugin_data *p) {
-	size_t key = 0, value = 0;
-	size_t is_key = 1, is_sid = 0;
-	size_t i;
-	
-	buffer_reset(p->session_id);
-	for (i = 0; i < con->uri.query->used; i++) {
-		switch(con->uri.query->ptr[i]) {
-		case '=':
-			if (is_key) {
-				if (0 == strncmp(con->uri.query->ptr + key, "PHPSESSID", i - key)) {
-					/* found PHP-session-id-key */
-						is_sid = 1;
-				}
-				value = i + 1;
-				
-				is_key = 0;
-			}
-			
-			break;
-		case '&':
-			if (is_sid) {
-				buffer_copy_string_len(p->session_id, con->uri.query->ptr + value, i - value);
-			}
-			
-			is_sid = 0;
-			key = i + 1;
-			value = 0;
-			is_key = 1;
-			break;
-		case ' ':
-			if (is_key == 1 && key == i) key = i + 1;
-			if (is_key == 0 && value == i) value = i + 1;
-			break;
-		case '\0':
-			if (is_sid) {
-				buffer_copy_string_len(p->session_id, con->uri.query->ptr + value, i - value);
-			}
-			/* fin */
-			break;
-		}
-	}
-	
-	if (!buffer_is_empty(p->session_id)) {
-		log_error_write(srv, __FILE__, __LINE__, "sb", 
-				"Session-ID", p->session_id);
-	}
-	
-	return !buffer_is_empty(p->session_id);
-}
-
-int cache_get_session_id(server *srv, connection *con, plugin_data *p) {
-	
-	return cache_get_cookie_session_id(srv, con, p) || 
-		cache_get_url_session_id(srv, con, p);
-	
-}
-
-
-URIHANDLER_FUNC(mod_cml_is_handled) {
-	int ct_len, s_len;
+int cache_call_lua(server *srv, connection *con, plugin_data *p, buffer *cml_file) {
 	buffer *b;
 	char *c;
-	buffer *fn = con->physical.path;
-	plugin_data *p = p_d;
 	int ret;
-	
-	if (fn->used == 0) return HANDLER_ERROR;
-	
-	mod_cml_patch_connection(srv, con, p);
-	
-	buffer_reset(p->basedir);
-	buffer_reset(p->baseurl);
-	buffer_reset(p->session_id);
-	buffer_reset(p->trigger_handler);
-	
-	if (buffer_is_empty(p->conf.ext)) return HANDLER_GO_ON;
-	
-	ct_len = p->conf.ext->used - 1;
-	s_len = fn->used - 1;
-	
-	if (s_len < ct_len) return HANDLER_GO_ON;
-	
-	if (0 != strncmp(fn->ptr + s_len - ct_len, p->conf.ext->ptr, ct_len)) {
-		/* not my job */
-		return HANDLER_GO_ON;
-	}
-	
+
 	/* cleanup basedir */
 	b = p->baseurl;
 	buffer_copy_string_buffer(b, con->uri.path);
@@ -337,7 +194,7 @@ URIHANDLER_FUNC(mod_cml_is_handled) {
 	}
 	
 	b = p->basedir;
-	buffer_copy_string_buffer(b, fn);
+	buffer_copy_string_buffer(b, con->physical.path);
 	for (c = b->ptr + b->used - 1; c > b->ptr && *c != '/'; c--);
 	
 	if (*c == '/') {
@@ -345,17 +202,86 @@ URIHANDLER_FUNC(mod_cml_is_handled) {
 		*(c+1) = '\0';
 	}
 	
+
 	/* prepare variables
-	 * - session-id
 	 *   - cookie-based
 	 *   - get-param-based
 	 */
 	
-	cache_get_session_id(srv, con, p);
+	return cache_parse_lua(srv, con, p, cml_file);
 	
-	ret = cache_parse_lua(srv, con, p, fn);
+}
+
+URIHANDLER_FUNC(mod_cml_power_magnet) {
+	plugin_data *p = p_d;
 	
-	switch(ret) {
+	mod_cml_patch_connection(srv, con, p);
+	
+	buffer_reset(p->basedir);
+	buffer_reset(p->baseurl);
+	buffer_reset(p->trigger_handler);
+
+	if (buffer_is_empty(p->conf.power_magnet)) return HANDLER_GO_ON;
+	
+	/* 
+	 * power-magnet:
+	 * cml.power-magnet = server.docroot + "/rewrite.cml"
+	 *
+	 * is called on EACH request, take the original REQUEST_URI and modifies the
+	 * request header as neccesary. 
+	 *
+	 * First use:
+	 * if file_exists("/maintainance.html") {
+	 *   output_include = ( "/maintainance.html" )
+	 *   return CACHE_HIT 
+	 * }
+	 *
+	 * as we only want to rewrite HTML like requests we should cover it in a conditional
+	 * 
+	 * */
+
+	switch(cache_call_lua(srv, con, p, p->conf.power_magnet)) {
+	case -1:
+		/* error */
+		if (con->conf.log_request_handling) {
+			log_error_write(srv, __FILE__, __LINE__, "s", "cache-error");
+		}
+		con->http_status = 500;
+		return HANDLER_COMEBACK;
+	case 0:
+		if (con->conf.log_request_handling) {
+			log_error_write(srv, __FILE__, __LINE__, "s", "cache-hit");
+		}
+		/* cache-hit */
+		buffer_reset(con->physical.path);
+		return HANDLER_FINISHED;
+	case 1:
+		/* cache miss */
+		return HANDLER_GO_ON;
+	default:
+		con->http_status = 500;
+		return HANDLER_COMEBACK;
+	}
+}
+
+URIHANDLER_FUNC(mod_cml_is_handled) {
+	plugin_data *p = p_d;
+	
+	if (buffer_is_empty(con->physical.path)) return HANDLER_ERROR;
+	
+	mod_cml_patch_connection(srv, con, p);
+	
+	buffer_reset(p->basedir);
+	buffer_reset(p->baseurl);
+	buffer_reset(p->trigger_handler);
+
+	if (buffer_is_empty(p->conf.ext)) return HANDLER_GO_ON;
+	
+	if (!buffer_is_equal_right_len(con->physical.path, p->conf.ext, p->conf.ext->used - 1)) {
+		return HANDLER_GO_ON;
+	} 
+
+	switch(cache_call_lua(srv, con, p, con->physical.path)) {
 	case -1:
 		/* error */
 		if (con->conf.log_request_handling) {
@@ -376,9 +302,10 @@ URIHANDLER_FUNC(mod_cml_is_handled) {
 		}
 		/* cache miss */
 		return HANDLER_COMEBACK;
+	default:
+		con->http_status = 500;
+		return HANDLER_COMEBACK;
 	}
-	
-	return 0;
 }
 
 int mod_cml_plugin_init(plugin *p) {
@@ -390,6 +317,7 @@ int mod_cml_plugin_init(plugin *p) {
 	p->set_defaults  = mod_cml_set_defaults;
 	
 	p->handle_subrequest_start = mod_cml_is_handled;
+	p->handle_physical         = mod_cml_power_magnet;
 	
 	p->data        = NULL;
 	
