@@ -23,7 +23,6 @@
 
 #include "inet_ntop_cache.h"
 #include "crc32.h"
-#include "network_backends.h"
 
 #include <stdio.h>
 
@@ -277,7 +276,6 @@ SETDEFAULTS_FUNC(mod_proxy_set_defaults) {
 					config_values_t pcv[] = { 
 						{ "host",              NULL, T_CONFIG_STRING, T_CONFIG_SCOPE_CONNECTION },      /* 0 */
 						{ "port",              NULL, T_CONFIG_SHORT, T_CONFIG_SCOPE_CONNECTION },       /* 1 */
-						{ "balance",              NULL, T_CONFIG_STRING, T_CONFIG_SCOPE_CONNECTION },   /* 2 */
 						{ NULL,                NULL, T_CONFIG_UNSET, T_CONFIG_SCOPE_UNSET }
 					};
 					
@@ -397,12 +395,36 @@ static int proxy_establish_connection(server *srv, handler_ctx *hctx) {
 	return 0;
 }
 
+void proxy_set_header(connection *con, const char *key, const char *value) {
+    data_string *ds_dst;
+
+    if (NULL == (ds_dst = (data_string *)array_get_unused_element(con->request.headers, TYPE_STRING))) {
+          ds_dst = data_string_init();
+    }
+
+    buffer_copy_string(ds_dst->key, key);
+    buffer_copy_string(ds_dst->value, value);
+    array_insert_unique(con->request.headers, (data_unset *)ds_dst);
+}
+
+void proxy_append_header(connection *con, const char *key, const char *value) {
+    data_string *ds_dst;
+
+    if (NULL == (ds_dst = (data_string *)array_get_unused_element(con->request.headers, TYPE_STRING))) {
+          ds_dst = data_string_init();
+    }
+
+    buffer_copy_string(ds_dst->key, key);
+    buffer_append_string(ds_dst->value, value);
+    array_insert_unique(con->request.headers, (data_unset *)ds_dst);
+}
+
+
 static int proxy_create_env(server *srv, handler_ctx *hctx) {
 	size_t i;
 	
 	connection *con   = hctx->remote_conn;
 	buffer *b;
-	data_string *ds_dst;
 	
 	/* build header */
 
@@ -415,15 +437,15 @@ static int proxy_create_env(server *srv, handler_ctx *hctx) {
 	buffer_append_string_buffer(b, con->request.uri);
 	BUFFER_APPEND_STRING_CONST(b, " HTTP/1.0\r\n");
 
-	if (NULL == (ds_dst = (data_string *)array_get_unused_element(con->request.headers, TYPE_STRING))) {
-		ds_dst = data_string_init();
+	proxy_append_header(con, "X-Forwarded-For", (char *)inet_ntop_cache_get_ip(srv, &(con->dst_addr)));
+	/* http_host is NOT is just a pointer to a buffer 
+	 * which is NULL if it is not set */
+	if (con->request.http_host && 
+	    !buffer_is_empty(con->request.http_host)) {
+		proxy_set_header(con, "X-Host", con->request.http_host->ptr);
 	}
-		
-	BUFFER_COPY_STRING_CONST(ds_dst->key, "X-Forwarded-For");
-	buffer_append_string(ds_dst->value, inet_ntop_cache_get_ip(srv, &(con->dst_addr)));
-		
-	array_insert_unique(con->request.headers, (data_unset *)ds_dst);
-	
+	proxy_set_header(con, "X-Forwarded-Proto", con->conf.is_ssl ? "https" : "http");
+
 	/* request header */
 	for (i = 0; i < con->request.headers->used; i++) {
 		data_string *ds;
@@ -431,7 +453,7 @@ static int proxy_create_env(server *srv, handler_ctx *hctx) {
 		ds = (data_string *)con->request.headers->data[i];
 		
 		if (ds->value->used && ds->key->used) {
-			if (0 == strcmp(ds->key->ptr, "Connection")) continue;
+			if (buffer_is_equal_string(ds->key, CONST_STR_LEN("Connection"))) continue;
 			
 			buffer_append_string_buffer(b, ds->key);
 			BUFFER_APPEND_STRING_CONST(b, ": ");
@@ -448,12 +470,12 @@ static int proxy_create_env(server *srv, handler_ctx *hctx) {
 	if (con->request.content_length) {
 		chunkqueue *req_cq = con->request_content_queue;
 		chunk *req_c;
-		size_t offset;
+		off_t offset;
 
 		/* something to send ? */
 		for (offset = 0, req_c = req_cq->first; offset != req_cq->bytes_in; req_c = req_c->next) {
-			size_t weWant = req_cq->bytes_in - offset;
-			size_t weHave = 0;
+			off_t weWant = req_cq->bytes_in - offset;
+			off_t weHave = 0;
 
 			/* we announce toWrite octects
 			 * now take all the request_content chunk that we need to fill this request
@@ -523,6 +545,7 @@ static int proxy_response_parse(server *srv, connection *con, plugin_data *p, bu
 		char *key, *value;
 		int key_len;
 		data_string *ds;
+		int copy_header;
 		
 		ns[0] = '\0';
 		ns[1] = '\0';
@@ -557,14 +580,7 @@ static int proxy_response_parse(server *srv, connection *con, plugin_data *p, bu
 		/* strip WS */
 		while (*value == ' ' || *value == '\t') value++;
 		
-		
-		if (NULL == (ds = (data_string *)array_get_unused_element(con->response.headers, TYPE_STRING))) {
-			ds = data_response_init();
-		}
-		buffer_copy_string_len(ds->key, key, key_len);
-		buffer_copy_string(ds->value, value);
-			
-		array_insert_unique(con->response.headers, (data_unset *)ds);
+		copy_header = 1;
 		
 		switch(key_len) {
 		case 4:
@@ -579,8 +595,7 @@ static int proxy_response_parse(server *srv, connection *con, plugin_data *p, bu
 			break;
 		case 10:
 			if (0 == strncasecmp(key, "Connection", key_len)) {
-				con->response.keep_alive = (0 == strcasecmp(value, "Keep-Alive")) ? 1 : 0;
-				con->parsed_response |= HTTP_CONNECTION;
+				copy_header = 0;
 			}
 			break;
 		case 14:
@@ -592,6 +607,16 @@ static int proxy_response_parse(server *srv, connection *con, plugin_data *p, bu
 		default:
 			break;
 		}
+
+		if (copy_header) {
+			if (NULL == (ds = (data_string *)array_get_unused_element(con->response.headers, TYPE_STRING))) {
+				ds = data_response_init();
+			}
+			buffer_copy_string_len(ds->key, key, key_len);
+			buffer_copy_string(ds->value, value);
+			
+			array_insert_unique(con->response.headers, (data_unset *)ds);
+		}
 	}
 	
 	return 0;
@@ -789,17 +814,7 @@ static handler_t proxy_write_request(server *srv, handler_ctx *hctx) {
 		
 		/* fall through */
 	case PROXY_STATE_WRITE:;
-#if defined USE_LINUX_SENDFILE
-		ret = network_write_chunkqueue_linuxsendfile(srv, con, hctx->fd, hctx->wb); 
-#elif defined USE_FREEBSD_SENDFILE
-		ret = network_write_chunkqueue_freebsdsendfile(srv, con, hctx->fd, hctx->wb); 
-#elif defined USE_SOLARIS_SENDFILEV
-		ret = network_write_chunkqueue_solarissendfilev(srv, con, hctx->fd, hctx->wb); 
-#elif defined USE_WRITEV
-		ret = network_write_chunkqueue_writev(srv, con, hctx->fd, hctx->wb);
-#else
-		ret = network_write_chunkqueue_write(srv, con, hctx->fd, hctx->wb);
-#endif
+		ret = srv->network_backend_write(srv, con, hctx->fd, hctx->wb); 
 
 		chunkqueue_remove_finished_chunks(hctx->wb);
 
@@ -1037,6 +1052,7 @@ static handler_t mod_proxy_check_extension(server *srv, connection *con, void *p
 	plugin_data *p = p_d;
 	size_t s_len;
 	unsigned long last_max = ULONG_MAX;
+	int max_usage = INT_MAX;
 	int ndx = -1;
 	size_t k;
 	buffer *fn;
@@ -1108,7 +1124,7 @@ static handler_t mod_proxy_check_extension(server *srv, connection *con, void *p
 					"proxy - used hash balancing, hosts:", extension->value->used);
 		}
 
-		for (k = 0, ndx = -1, last_max = ~0L; k < extension->value->used; k++) {
+		for (k = 0, ndx = -1, last_max = ULONG_MAX; k < extension->value->used; k++) {
 			data_proxy *host = (data_proxy *)extension->value->data[k];
 			unsigned long cur_max;
 
@@ -1127,7 +1143,7 @@ static handler_t mod_proxy_check_extension(server *srv, connection *con, void *p
 						cur_max);
 			}
 
-			if ((last_max == ~0L) || /* first round */
+			if ((last_max == ULONG_MAX) || /* first round */
 		   	    (cur_max > last_max)) {
 				last_max = cur_max;
 
@@ -1143,13 +1159,13 @@ static handler_t mod_proxy_check_extension(server *srv, connection *con, void *p
 					"proxy - used fair balancing");
 		}
 
-		for (k = 0, ndx = -1, last_max = ~0L; k < extension->value->used; k++) {
+		for (k = 0, ndx = -1, max_usage = INT_MAX; k < extension->value->used; k++) {
 			data_proxy *host = (data_proxy *)extension->value->data[k];
 		
 			if (host->is_disabled) continue;
 
-			if (host->usage < last_max) {
-				last_max = host->usage;
+			if (host->usage < max_usage) {
+				max_usage = host->usage;
 			
 				ndx = k;
 			}
@@ -1162,18 +1178,22 @@ static handler_t mod_proxy_check_extension(server *srv, connection *con, void *p
 			log_error_write(srv, __FILE__, __LINE__,  "s", 
 					"proxy - used round-robin balancing");
 		}
-		for (k = 0, ndx = -1, last_max = ~0L; k < extension->value->used; k++) {
+
+		/* just to be sure */
+		assert(extension->value->used < INT_MAX);
+		
+		for (k = 0, ndx = -1, max_usage = INT_MAX; k < extension->value->used; k++) {
 			data_proxy *host = (data_proxy *)extension->value->data[k];
 		
 			if (host->is_disabled) continue;
 
 			/* first usable ndx */
-			if (last_max == ~0L) {
-				last_max = k;
+			if (max_usage == INT_MAX) {
+				max_usage = k;
 			}
 
 			/* get next ndx */
-			if (k > host->last_used_ndx) {
+			if ((int)k > host->last_used_ndx) {
 				ndx = k;
 				host->last_used_ndx = k;
 
@@ -1182,8 +1202,8 @@ static handler_t mod_proxy_check_extension(server *srv, connection *con, void *p
 		}
 		
 		/* didn't found a higher id, wrap to the start */
-		if (ndx != -1 && last_max != ~0L) {
-			ndx = last_max;
+		if (ndx != -1 && max_usage != INT_MAX) {
+			ndx = max_usage;
 		}
 
 		break;
