@@ -22,7 +22,6 @@
 #include "plugin.h"
 
 #include "inet_ntop_cache.h"
-#include "network_backends.h"
 
 #include <stdio.h>
 
@@ -746,6 +745,7 @@ static int scgi_spawn_connection(server *srv,
 		case 0: {
 			buffer *b;
 			size_t i = 0;
+			int fd = 0;
 			char_array env;
 			
 			
@@ -755,8 +755,8 @@ static int scgi_spawn_connection(server *srv,
 			env.used = 0;
 			
 			/* we don't need the client socket */
-			for (i = 3; i < 256; i++) {
-				if (i != 2 && i != scgi_fd) close(i);
+			for (fd = 3; fd < 256; fd++) {
+				if (fd != 2 && fd != scgi_fd) close(fd);
 			}
 			
 			/* build clean environment */
@@ -1578,12 +1578,12 @@ static int scgi_create_env(server *srv, handler_ctx *hctx) {
 	if (con->request.content_length) {
 		chunkqueue *req_cq = con->request_content_queue;
 		chunk *req_c;
-		size_t offset;
+		off_t offset;
 
 		/* something to send ? */
 		for (offset = 0, req_c = req_cq->first; offset != req_cq->bytes_in; req_c = req_c->next) {
-			size_t weWant = req_cq->bytes_in - offset;
-			size_t weHave = 0;
+			off_t weWant = req_cq->bytes_in - offset;
+			off_t weHave = 0;
 
 			/* we announce toWrite octects
 			 * now take all the request_content chunk that we need to fill this request
@@ -2257,18 +2257,7 @@ static handler_t scgi_write_request(server *srv, handler_ctx *hctx) {
 		
 		/* fall through */
 	case FCGI_STATE_WRITE:
-		/* why aren't we using the network_ interface here ? */
-#if defined USE_LINUX_SENDFILE
-		ret = network_write_chunkqueue_linuxsendfile(srv, con, hctx->fd, hctx->wb); 
-#elif defined USE_FREEBSD_SENDFILE
-		ret = network_write_chunkqueue_freebsdsendfile(srv, con, hctx->fd, hctx->wb); 
-#elif defined USE_SOLARIS_SENDFILEV
-		ret = network_write_chunkqueue_solarissendfilev(srv, con, hctx->fd, hctx->wb); 
-#elif defined USE_WRITEV
-		ret = network_write_chunkqueue_writev(srv, con, hctx->fd, hctx->wb);
-#else
-		ret = network_write_chunkqueue_write(srv, con, hctx->fd, hctx->wb);
-#endif
+		ret = srv->network_backend_write(srv, con, hctx->fd, hctx->wb); 
 
 		chunkqueue_remove_finished_chunks(hctx->wb);
 	
@@ -2688,11 +2677,9 @@ static handler_t scgi_check_extension(server *srv, connection *con, void *p_d, i
 	/* Possibly, we processed already this request */
 	if (con->file_started == 1) return HANDLER_GO_ON;
 	
-	fn = con->uri.path;
+	fn = uri_path_handler ? con->uri.path : con->physical.path;
 
-	if (fn->used == 0) {
-		return HANDLER_ERROR;
-	}
+	if (buffer_is_empty(fn)) return HANDLER_GO_ON;
 
 	s_len = fn->used - 1;
 	
