@@ -372,17 +372,11 @@ URIHANDLER_FUNC(mod_staticfile_subrequest) {
 	
 	/* ignore certain extensions */
 	for (k = 0; k < p->conf.exclude_ext->used; k++) {
-		int ct_len;
-
 		ds = (data_string *)p->conf.exclude_ext->data[k]; 
-
-		ct_len = ds->value->used - 1;
-		
-		if (ct_len > s_len) continue;
 		
 		if (ds->value->used == 0) continue;
-		
-		if (0 == strncmp(con->uri.path->ptr + s_len - ct_len, ds->value->ptr, ct_len)) {
+
+		if (buffer_is_equal_right_len(con->physical.path, ds->value, ds->value->used - 1)) {
 			return HANDLER_GO_ON;
 		}
 	}
@@ -446,13 +440,27 @@ URIHANDLER_FUNC(mod_staticfile_subrequest) {
 	if (HANDLER_FINISHED == http_response_handle_cachable(srv, con, mtime)) {
 		return HANDLER_FINISHED;
 	} else if (con->request.http_range && con->conf.range_requests) {
-		/* content prepared, I'm done */
-		con->file_finished = 1;
+		int do_range_request = 1;
+		/* check if we have a conditional GET */
+
+		if (NULL != (ds = (data_string *)array_get_element(con->request.headers, "If-Range"))) {
+			/* if the value is the same as our ETag, we do a Range-request, 
+			 * otherwise a full 200 */
+
+			if (!buffer_is_equal(ds->value, con->physical.etag)) {
+				do_range_request = 0;
+			}
+		}
+	
+		if (do_range_request) {
+			/* content prepared, I'm done */
+			con->file_finished = 1;
 		
-		if (0 == http_response_parse_range(srv, con, p)) {
-			con->http_status = 206;
+			if (0 == http_response_parse_range(srv, con, p)) {
+				con->http_status = 206;
+			}
+			return HANDLER_FINISHED;
 		}
-		return HANDLER_FINISHED;
 	}
 	
 	/* if we are still here, prepare body */
