@@ -49,6 +49,7 @@
 typedef struct {
 	unsigned short enabled;
 	unsigned short is_readonly;
+	unsigned short log_xml;
 
 	buffer *sqlite_db_name;
 #ifdef USE_PROPPATCH
@@ -157,6 +158,7 @@ SETDEFAULTS_FUNC(mod_webdav_set_defaults) {
 		{ "webdav.activate",            NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_CONNECTION },       /* 0 */
 		{ "webdav.is-readonly",         NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_CONNECTION },       /* 1 */
 		{ "webdav.sqlite-db-name",      NULL, T_CONFIG_STRING,  T_CONFIG_SCOPE_CONNECTION },       /* 2 */
+		{ "webdav.log-xml",             NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_CONNECTION },       /* 3 */
 		{ NULL,                         NULL, T_CONFIG_UNSET, T_CONFIG_SCOPE_UNSET }
 	};
 	
@@ -173,6 +175,7 @@ SETDEFAULTS_FUNC(mod_webdav_set_defaults) {
 		cv[0].destination = &(s->enabled);
 		cv[1].destination = &(s->is_readonly);
 		cv[2].destination = s->sqlite_db_name;
+		cv[3].destination = &(s->log_xml);
 		
 		p->config_storage[i] = s;
 	
@@ -288,6 +291,7 @@ static int mod_webdav_patch_connection(server *srv, connection *con, plugin_data
 	
 	PATCH(enabled);
 	PATCH(is_readonly);
+	PATCH(log_xml);
 	
 #ifdef USE_PROPPATCH
 	PATCH(sql);
@@ -316,6 +320,8 @@ static int mod_webdav_patch_connection(server *srv, connection *con, plugin_data
 				PATCH(enabled);
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("webdav.is-readonly"))) {
 				PATCH(is_readonly);
+			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("webdav.log-xml"))) {
+				PATCH(log_xml);
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("webdav.sqlite-db-name"))) {
 #ifdef USE_PROPPATCH
 				PATCH(sql);
@@ -447,7 +453,6 @@ static int webdav_delete_file(server *srv, connection *con, plugin_data *p, phys
 		if (!stmt) {
 			status = 403;
 			webdav_gen_response_status_tag(srv, con, dst, status, b);
-
 		} else {
 			sqlite3_reset(stmt);
 
@@ -525,18 +530,20 @@ static int webdav_delete_dir(server *srv, connection *con, plugin_data *p, physi
 
 					status = 0;
 
-					sqlite3_reset(stmt);
+					if (stmt) {
+						sqlite3_reset(stmt);
 
-					/* bind the values to the insert */
+						/* bind the values to the insert */
 
-					sqlite3_bind_text(stmt, 1, 
-							  d.rel_path->ptr, 
-							  d.rel_path->used - 1,
-							  SQLITE_TRANSIENT);
+						sqlite3_bind_text(stmt, 1, 
+								  d.rel_path->ptr, 
+								  d.rel_path->used - 1,
+								  SQLITE_TRANSIENT);
 													
-					if (SQLITE_DONE != sqlite3_step(stmt)) {
-						/* */
-						WP();
+						if (SQLITE_DONE != sqlite3_step(stmt)) {
+							/* */
+							WP();
+						}
 					}
 #endif
 				}
@@ -603,22 +610,24 @@ static int webdav_copy_file(server *srv, connection *con, plugin_data *p, physic
 		/* copy worked fine, copy connected properties */
 		sqlite3_stmt *stmt = p->conf.stmt_copy_uri;
 
-		sqlite3_reset(stmt);
+		if (stmt) {
+			sqlite3_reset(stmt);
 
-		/* bind the values to the insert */
-		sqlite3_bind_text(stmt, 1, 
-				  dst->rel_path->ptr, 
-				  dst->rel_path->used - 1,
-				  SQLITE_TRANSIENT);
+			/* bind the values to the insert */
+			sqlite3_bind_text(stmt, 1, 
+					  dst->rel_path->ptr, 
+					  dst->rel_path->used - 1,
+					  SQLITE_TRANSIENT);
 
-		sqlite3_bind_text(stmt, 2, 
-				  src->rel_path->ptr, 
-				  src->rel_path->used - 1,
-				  SQLITE_TRANSIENT);
+			sqlite3_bind_text(stmt, 2, 
+					  src->rel_path->ptr, 
+					  src->rel_path->used - 1,
+					  SQLITE_TRANSIENT);
 													
-		if (SQLITE_DONE != sqlite3_step(stmt)) {
-			/* */
-			WP();
+			if (SQLITE_DONE != sqlite3_step(stmt)) {
+				/* */
+				WP();
+			}
 		}
 	}
 #endif
@@ -678,23 +687,25 @@ static int webdav_copy_dir(server *srv, connection *con, plugin_data *p, physica
 						break;
 					}
 					/* directory is copied, copy the properties too */
-				
-					sqlite3_reset(stmt);
 
-					/* bind the values to the insert */
-					sqlite3_bind_text(stmt, 1, 
-						  dst->rel_path->ptr, 
-						  dst->rel_path->used - 1,
-						  SQLITE_TRANSIENT);
+					if (stmt) {
+						sqlite3_reset(stmt);
 
-					sqlite3_bind_text(stmt, 2, 
-						  src->rel_path->ptr, 
-						  src->rel_path->used - 1,
-						  SQLITE_TRANSIENT);
+						/* bind the values to the insert */
+						sqlite3_bind_text(stmt, 1, 
+							  dst->rel_path->ptr, 
+							  dst->rel_path->used - 1,
+							  SQLITE_TRANSIENT);
+
+						sqlite3_bind_text(stmt, 2, 
+							  src->rel_path->ptr, 
+							  src->rel_path->used - 1,
+							  SQLITE_TRANSIENT);
 													
-					if (SQLITE_DONE != sqlite3_step(stmt)) {
-						/* */
-						WP();
+						if (SQLITE_DONE != sqlite3_step(stmt)) {
+							/* */
+							WP();
+						}
 					}
 #endif
 				}
@@ -721,6 +732,8 @@ static int webdav_get_live_property(server *srv, connection *con, plugin_data *p
 	stat_cache_entry *sce = NULL;
 	int found = 0;
 
+	UNUSED(p);
+
 	if (HANDLER_ERROR != (stat_cache_get_entry(srv, con, dst->path, &sce))) {
 		char ctime_buf[] = "2005-08-18T07:27:16Z";
 		char mtime_buf[] = "Thu, 18 Aug 2005 07:27:16 GMT";
@@ -786,29 +799,33 @@ static int webdav_get_property(server *srv, connection *con, plugin_data *p, phy
 	} else {
 		int found = 0;
 #ifdef USE_PROPPATCH
-		/* perhaps it is in sqlite3 */
-		sqlite3_reset(p->conf.stmt_select_prop);
+		sqlite3_stmt *stmt = p->conf.stmt_select_prop;
 
-		/* bind the values to the insert */
+		if (stmt) {
+			/* perhaps it is in sqlite3 */
+			sqlite3_reset(stmt);
 
-		sqlite3_bind_text(p->conf.stmt_select_prop, 1, 
-				  dst->rel_path->ptr, 
-				  dst->rel_path->used - 1,
-				  SQLITE_TRANSIENT);
-		sqlite3_bind_text(p->conf.stmt_select_prop, 2, 
-				  prop_name,
-				  strlen(prop_name),
-				  SQLITE_TRANSIENT);
-		sqlite3_bind_text(p->conf.stmt_select_prop, 3, 
-				  prop_ns,
-				  strlen(prop_ns),
-				  SQLITE_TRANSIENT);
+			/* bind the values to the insert */
 
-		/* it is the PK */
-		while (SQLITE_ROW == sqlite3_step(p->conf.stmt_select_prop)) {
-			/* there is a row for us, we only expect a single col 'value' */
-			webdav_gen_prop_tag(srv, con, prop_name, prop_ns, (char *)sqlite3_column_text(p->conf.stmt_select_prop, 0), b);
-			found = 1;
+			sqlite3_bind_text(stmt, 1, 
+					  dst->rel_path->ptr, 
+					  dst->rel_path->used - 1,
+					  SQLITE_TRANSIENT);
+			sqlite3_bind_text(stmt, 2, 
+					  prop_name,
+					  strlen(prop_name),
+					  SQLITE_TRANSIENT);
+			sqlite3_bind_text(stmt, 3, 
+					  prop_ns,
+					  strlen(prop_ns),
+					  SQLITE_TRANSIENT);
+
+			/* it is the PK */
+			while (SQLITE_ROW == sqlite3_step(p->conf.stmt_select_prop)) {
+				/* there is a row for us, we only expect a single col 'value' */
+				webdav_gen_prop_tag(srv, con, prop_name, prop_ns, (char *)sqlite3_column_text(p->conf.stmt_select_prop, 0), b);
+				found = 1;
+			}
 		}
 #endif
 		return found ? 0 : -1;
@@ -871,7 +888,7 @@ static int webdav_get_props(server *srv, connection *con, plugin_data *p, physic
 }
 
 #ifdef USE_PROPPATCH
-static int webdav_parse_chunkqueue(server *srv, connection *con, chunkqueue *cq, xmlDoc **ret_xml) {
+static int webdav_parse_chunkqueue(server *srv, connection *con, plugin_data *p, chunkqueue *cq, xmlDoc **ret_xml) {
 	xmlParserCtxtPtr ctxt;
 	xmlDoc *xml;
 	int res;
@@ -879,6 +896,8 @@ static int webdav_parse_chunkqueue(server *srv, connection *con, chunkqueue *cq,
 
 	chunk *c;
 
+	UNUSED(con);
+
 	/* read the chunks in to the XML document */
 	ctxt = xmlCreatePushParserCtxt(NULL, NULL, NULL, 0, NULL);
 
@@ -930,6 +949,10 @@ static int webdav_parse_chunkqueue(server *srv, connection *con, chunkqueue *cq,
 
 			if (weHave > weWant) weHave = weWant;
 
+			if (p->conf.log_xml) {
+				log_error_write(srv, __FILE__, __LINE__, "ss", "XML-request-body:", c->mem->ptr + c->offset);
+			}
+
 			if (XML_ERR_OK != (err = xmlParseChunk(ctxt, c->mem->ptr + c->offset, weHave, 0))) {
 				log_error_write(srv, __FILE__, __LINE__, "sddd", "xmlParseChunk failed at:", cq->bytes_out, weHave, err);
 			}
@@ -937,14 +960,21 @@ static int webdav_parse_chunkqueue(server *srv, connection *con, chunkqueue *cq,
 			c->offset += weHave;
 			cq->bytes_out += weHave;
 
+			break;
+		case UNUSED_CHUNK:
 			break;
 		}
 		chunkqueue_remove_finished_chunks(cq);
 	}
 
 
-	if (XML_ERR_DOCUMENT_END != (err = xmlParseChunk(ctxt, 0, 0, 1))) {
+	switch ((err = xmlParseChunk(ctxt, 0, 0, 1))) {
+	case XML_ERR_DOCUMENT_END:
+	case XML_ERR_OK:
+		break;
+	default:
 		log_error_write(srv, __FILE__, __LINE__, "sd", "xmlParseChunk failed at final packet:", err);
+		break;
 	}
 
 	xml = ctxt->myDoc;
@@ -995,7 +1025,7 @@ URIHANDLER_FUNC(mod_webdav_subrequest_handler) {
 		if (con->request.content_length) {
 			xmlDocPtr xml;
 
-			if (1 == webdav_parse_chunkqueue(srv, con, con->request_content_queue, &xml)) {
+			if (1 == webdav_parse_chunkqueue(srv, con, p, con->request_content_queue, &xml)) {
 				xmlNode *rootnode = xmlDocGetRootElement(xml);
 
 				assert(rootnode);
@@ -1051,19 +1081,22 @@ URIHANDLER_FUNC(mod_webdav_subrequest_handler) {
 								req_props->used++;
 							}
 						} else if (0 == xmlStrcmp(cmd->name, BAD_CAST "propname")) {
-							/* get all property names (EMPTY) */
-							sqlite3_reset(p->conf.stmt_select_propnames);
-							/* bind the values to the insert */
+							sqlite3_stmt *stmt = p->conf.stmt_select_propnames;
 
-							sqlite3_bind_text(p->conf.stmt_select_propnames, 1, 
-									  con->uri.path->ptr, 
-									  con->uri.path->used - 1,
-									  SQLITE_TRANSIENT);
+							if (stmt) {
+								/* get all property names (EMPTY) */
+								sqlite3_reset(stmt);
+								/* bind the values to the insert */
+	
+								sqlite3_bind_text(stmt, 1, 
+										  con->uri.path->ptr, 
+										  con->uri.path->used - 1,
+										  SQLITE_TRANSIENT);
 						
-							if (SQLITE_DONE != sqlite3_step(p->conf.stmt_select_propnames)) {
-								WP();
+								if (SQLITE_DONE != sqlite3_step(stmt)) {
+									WP();
+								}
 							}
-
 						} else if (0 == xmlStrcmp(cmd->name, BAD_CAST "allprop")) {
 							/* get all properties (EMPTY) */
 						}
@@ -1102,7 +1135,7 @@ URIHANDLER_FUNC(mod_webdav_subrequest_handler) {
 			buffer_append_string_buffer(b, con->uri.scheme);
 			buffer_append_string(b,"://");
 			buffer_append_string_buffer(b, con->uri.authority);
-			buffer_append_string_buffer(b, con->uri.path);
+			buffer_append_string_encoded(b, CONST_BUF_LEN(con->uri.path), ENCODING_REL_URI);
 			buffer_append_string(b,"</D:href>\n");
 
 			if (!buffer_is_empty(prop_200)) {
@@ -1143,19 +1176,23 @@ URIHANDLER_FUNC(mod_webdav_subrequest_handler) {
 				d.rel_path = buffer_init();
 
 				while(NULL != (de = readdir(dir))) {
-					if ((de->d_name[0] == '.' && de->d_name[1] == '\0')  ||
-					    (de->d_name[0] == '.' && de->d_name[1] == '.' && de->d_name[2] == '\0')) {
+					if (de->d_name[0] == '.' && de->d_name[1] == '.' && de->d_name[2] == '\0') {
 						continue;
 						/* ignore the parent dir */
 					} 
 
 					buffer_copy_string_buffer(d.path, dst->path);
 					BUFFER_APPEND_SLASH(d.path);
-					buffer_append_string(d.path, de->d_name);
-			
+
 					buffer_copy_string_buffer(d.rel_path, dst->rel_path);
 					BUFFER_APPEND_SLASH(d.rel_path);
-					buffer_append_string(d.rel_path, de->d_name);
+
+					if (de->d_name[0] == '.' && de->d_name[1] == '\0') {
+						/* don't append the . */ 
+					} else {
+						buffer_append_string(d.path, de->d_name);
+						buffer_append_string(d.rel_path, de->d_name);
+					}
 
 					buffer_reset(prop_200);
 					buffer_reset(prop_404);
@@ -1167,7 +1204,7 @@ URIHANDLER_FUNC(mod_webdav_subrequest_handler) {
 					buffer_append_string_buffer(b, con->uri.scheme);
 					buffer_append_string(b,"://");
 					buffer_append_string_buffer(b, con->uri.authority);
-					buffer_append_string_buffer(b, d.rel_path);
+					buffer_append_string_encoded(b, CONST_BUF_LEN(d.rel_path), ENCODING_REL_URI);
 					buffer_append_string(b,"</D:href>\n");
 
 					if (!buffer_is_empty(prop_200)) {
@@ -1215,8 +1252,14 @@ URIHANDLER_FUNC(mod_webdav_subrequest_handler) {
 			free(req_props);
 		}
 
+		buffer_free(prop_200);
+		buffer_free(prop_404);
+
 		buffer_append_string(b,"</D:multistatus>\n");
 
+		if (p->conf.log_xml) {
+			log_error_write(srv, __FILE__, __LINE__, "sb", "XML-response-body:", b);
+		}
 		con->file_finished = 1;
 
 		return HANDLER_FINISHED;
@@ -1288,6 +1331,10 @@ URIHANDLER_FUNC(mod_webdav_subrequest_handler) {
 
 				buffer_append_string(b,"</D:multistatus>\n");
 			
+				if (p->conf.log_xml) {
+					log_error_write(srv, __FILE__, __LINE__, "sb", "XML-response-body:", b);
+				}
+
 				con->http_status = 207;
 				con->file_finished = 1;
 			} else {
@@ -1333,7 +1380,7 @@ URIHANDLER_FUNC(mod_webdav_subrequest_handler) {
 			return HANDLER_FINISHED;
 		}
 
-		assert(chunkqueue_length(cq) == con->request.content_length);
+		assert(chunkqueue_length(cq) == (off_t)con->request.content_length);
 
 		/* taken what we have in the request-body and write it to a file */
 		if (-1 == (fd = open(con->physical.path->ptr, O_WRONLY|O_CREAT|O_TRUNC, 0600))) {
@@ -1399,6 +1446,8 @@ URIHANDLER_FUNC(mod_webdav_subrequest_handler) {
 						}
 					}
 					break;
+				case UNUSED_CHUNK:
+					break;
 				}
 
 				if (r > 0) {
@@ -1490,10 +1539,10 @@ URIHANDLER_FUNC(mod_webdav_subrequest_handler) {
 		buffer_path_simplify(p->uri.path, p->tmp_buf);
 
 		/* we now have a URI which is clean. transform it into a physical path */
-		buffer_copy_string_buffer(p->physical.doc_root, con->conf.document_root);
+		buffer_copy_string_buffer(p->physical.doc_root, con->physical.doc_root);
 		buffer_copy_string_buffer(p->physical.rel_path, p->uri.path);
 
-		if (con->conf.force_lower_case) {
+		if (con->conf.force_lowercase_filenames) {
 			buffer_to_lower(p->physical.rel_path);
 		}
 
@@ -1597,21 +1646,24 @@ URIHANDLER_FUNC(mod_webdav_subrequest_handler) {
 #ifdef USE_PROPPATCH
 					sqlite3_stmt *stmt = p->conf.stmt_move_uri;
 
-					sqlite3_reset(stmt);
+					if (stmt) {
 
-					/* bind the values to the insert */
-					sqlite3_bind_text(stmt, 1, 
-							  p->uri.path->ptr, 
-							  p->uri.path->used - 1,
-							  SQLITE_TRANSIENT);
+						sqlite3_reset(stmt);
 
-					sqlite3_bind_text(stmt, 2, 
-							  con->uri.path->ptr, 
-							  con->uri.path->used - 1,
-							  SQLITE_TRANSIENT);
-					
-					if (SQLITE_DONE != sqlite3_step(stmt)) {
-						log_error_write(srv, __FILE__, __LINE__, "ss", "sql-move failed:", sqlite3_errmsg(p->conf.sql));
+						/* bind the values to the insert */
+						sqlite3_bind_text(stmt, 1, 
+								  p->uri.path->ptr, 
+								  p->uri.path->used - 1,
+								  SQLITE_TRANSIENT);
+
+						sqlite3_bind_text(stmt, 2, 
+								  con->uri.path->ptr, 
+								  con->uri.path->used - 1,
+								  SQLITE_TRANSIENT);
+						
+						if (SQLITE_DONE != sqlite3_step(stmt)) {
+							log_error_write(srv, __FILE__, __LINE__, "ss", "sql-move failed:", sqlite3_errmsg(p->conf.sql));
+						}
 					}
 #endif
 					return HANDLER_FINISHED;
@@ -1654,7 +1706,7 @@ URIHANDLER_FUNC(mod_webdav_subrequest_handler) {
 		if (con->request.content_length) {
 			xmlDocPtr xml;
 
-			if (1 == webdav_parse_chunkqueue(srv, con, con->request_content_queue, &xml)) {
+			if (1 == webdav_parse_chunkqueue(srv, con, p, con->request_content_queue, &xml)) {
 				xmlNode *rootnode = xmlDocGetRootElement(xml);
 
 				if (0 == xmlStrcmp(rootnode->name, BAD_CAST "propertyupdate")) {
