@@ -70,6 +70,8 @@ int network_server_init(server *srv, buffer *host_token, specific_config *s) {
 	unsigned int port = 0;
 	const char *host;
 	buffer *b;
+	int is_unix_domain_socket = 0;
+	int fd;
 	
 #ifdef SO_ACCEPTFILTER
 	struct accept_filter_arg afa;
@@ -121,15 +123,34 @@ int network_server_init(server *srv, buffer *host_token, specific_config *s) {
 	*(sp++) = '\0';
 	
 	port = strtol(sp, NULL, 10);
-	
-	if (port == 0 || port > 65535) {
+
+	if (host[0] == '/') {
+		/* host is a unix-domain-socket */
+		is_unix_domain_socket = 1;
+	} else if (port == 0 || port > 65535) {
 		log_error_write(srv, __FILE__, __LINE__, "sd", "port out of range:", port);
 	
 		return -1;
 	}
 	
 	if (*host == '\0') host = NULL;
-	
+
+	if (is_unix_domain_socket) {
+#ifdef HAVE_SYS_UN_H
+
+		srv_socket->addr.plain.sa_family = AF_UNIX;
+		
+		if (-1 == (srv_socket->fd = socket(srv_socket->addr.plain.sa_family, SOCK_STREAM, 0))) {
+			log_error_write(srv, __FILE__, __LINE__, "ss", "socket failed:", strerror(errno));
+			return -1;
+		}
+#else
+		log_error_write(srv, __FILE__, __LINE__, "s",
+				"ERROR: Unix Domain sockets are not supported.");
+		return -1;
+#endif
+	}
+
 #ifdef HAVE_IPV6
 	if (s->use_ipv6) {
 		srv_socket->addr.plain.sa_family = AF_INET6;
@@ -159,11 +180,6 @@ int network_server_init(server *srv, buffer *host_token, specific_config *s) {
 		return -1;
 	}
 	
-	if (-1 == fdevent_fcntl_set(srv->ev, srv_socket->fd)) {
-		log_error_write(srv, __FILE__, __LINE__, "ss", "fcntl failed:", strerror(errno));
-		return -1;
-	}
-	
 	switch(srv_socket->addr.plain.sa_family) {
 #ifdef HAVE_IPV6
 	case AF_INET6:
@@ -227,6 +243,45 @@ int network_server_init(server *srv, buffer *host_token, specific_config *s) {
 		
 		addr_len = sizeof(struct sockaddr_in);
 		
+		break;
+	case AF_UNIX:
+		srv_socket->addr.un.sun_family = AF_UNIX;
+		strcpy(srv_socket->addr.un.sun_path, host);
+		
+#ifdef SUN_LEN
+		addr_len = SUN_LEN(&srv_socket->addr.un);
+#else
+		/* stevens says: */
+		addr_len = strlen(host) + sizeof(srv_socket->addr.un.sun_family);
+#endif
+
+		/* check if the socket exists and try to connect to it. */
+		if (-1 != (fd = connect(srv_socket->fd, (struct sockaddr *) &(srv_socket->addr), addr_len))) {
+			close(fd);
+
+			log_error_write(srv, __FILE__, __LINE__, "ss", 
+				"server socket is still in use:", 
+				host);
+
+
+			return -1;
+		}
+
+		/* connect failed */
+		switch(errno) {
+		case ECONNREFUSED:
+			unlink(host);
+			break;
+		case ENOENT:
+			break;
+		default:
+			log_error_write(srv, __FILE__, __LINE__, "sds", 
+				"testing socket failed:", 
+				host, strerror(errno));
+
+			return -1;
+		}
+
 		break;
 	default:
 		addr_len = 0;
@@ -235,7 +290,18 @@ int network_server_init(server *srv, buffer *host_token, specific_config *s) {
 	}
 	
 	if (0 != bind(srv_socket->fd, (struct sockaddr *) &(srv_socket->addr), addr_len)) {
-		log_error_write(srv, __FILE__, __LINE__, "sds", "can't bind to port", port, strerror(errno));
+		switch(srv_socket->addr.plain.sa_family) {
+		case AF_UNIX:
+			log_error_write(srv, __FILE__, __LINE__, "sds", 
+					"can't bind to socket:", 
+					host, strerror(errno));
+			break;
+		default:
+			log_error_write(srv, __FILE__, __LINE__, "ssds", 
+					"can't bind to port:", 
+					host, port, strerror(errno));
+			break;
+		}
 		return -1;
 	}
 	
@@ -369,9 +435,40 @@ int network_close(server *srv) {
 	return 0;
 }
 
+typedef enum {
+	NETWORK_BACKEND_UNSET,
+	NETWORK_BACKEND_WRITE,
+	NETWORK_BACKEND_WRITEV,
+	NETWORK_BACKEND_LINUX_SENDFILE,
+	NETWORK_BACKEND_FREEBSD_SENDFILE,
+	NETWORK_BACKEND_SOLARIS_SENDFILEV
+} network_backend_t;
+
 int network_init(server *srv) {
 	buffer *b;
 	size_t i;
+	network_backend_t backend;
+	
+	struct nb_map { 
+		network_backend_t nb; 
+		const char *name; 
+	} network_backends[] = { 
+		/* lowest id wins */
+#if defined USE_LINUX_SENDFILE
+		{ NETWORK_BACKEND_LINUX_SENDFILE,       "linux-sendfile" },
+#endif
+#if defined USE_FREEBSD_SENDFILE
+		{ NETWORK_BACKEND_FREEBSD_SENDFILE,     "freebsd-sendfile" },
+#endif
+#if defined USE_SOLARIS_SENDFILEV
+		{ NETWORK_BACKEND_SOLARIS_SENDFILEV,	"solaris-sendfilev" },
+#endif
+#if defined USE_WRITEV
+		{ NETWORK_BACKEND_WRITEV,		"writev" },
+#endif
+		{ NETWORK_BACKEND_WRITE,		"write" },
+		{ NETWORK_BACKEND_UNSET,        	NULL }
+	};
 	
 	b = buffer_init();
 		
@@ -384,6 +481,61 @@ int network_init(server *srv) {
 	}
 	buffer_free(b);
 		
+#ifdef USE_OPENSSL
+	srv->network_ssl_backend_write = network_write_chunkqueue_openssl;
+#endif
+
+	/* get a usefull default */
+	backend = network_backends[0].nb;
+
+	/* match name against known types */
+	if (!buffer_is_empty(srv->srvconf.network_backend)) {
+		for (i = 0; network_backends[i].name; i++) {
+			/**/
+			if (buffer_is_equal_string(srv->srvconf.network_backend, network_backends[i].name, strlen(network_backends[i].name))) {
+				backend = network_backends[i].nb;
+				break;
+			}
+		}
+		if (NULL == network_backends[i].name) {
+			/* we don't know it */
+
+			log_error_write(srv, __FILE__, __LINE__, "sb", 
+					"server.network-backend has a unknown value:", 
+					srv->srvconf.network_backend);
+
+			return -1;
+		}
+	}
+
+	switch(backend) {
+	case NETWORK_BACKEND_WRITE:
+		srv->network_backend_write = network_write_chunkqueue_write;
+		break;
+#ifdef USE_WRITEV
+	case NETWORK_BACKEND_WRITEV:
+		srv->network_backend_write = network_write_chunkqueue_writev;
+		break;
+#endif
+#ifdef USE_LINUX_SENDFILE
+	case NETWORK_BACKEND_LINUX_SENDFILE:
+		srv->network_backend_write = network_write_chunkqueue_linuxsendfile; 
+		break;
+#endif
+#ifdef USE_FREEBSD_SENDFILE
+	case NETWORK_BACKEND_FREEBSD_SENDFILE:
+		srv->network_backend_write = network_write_chunkqueue_freebsdsendfile; 
+		break;
+#endif
+#ifdef USE_SOLARIS_SENDFILEV
+	case NETWORK_BACKEND_SOLARIS_SENDFILEV:
+		srv->network_backend_write = network_write_chunkqueue_solarissendfilev; 
+		break;
+#endif
+	default:
+		return -1;
+	}
+
 	/* check for $SERVER["socket"] */
 	for (i = 1; i < srv->config_context->used; i++) {
 		data_config *dc = (data_config *)srv->config_context->data[i];
@@ -456,21 +608,10 @@ int network_write_chunkqueue(server *srv, connection *con, chunkqueue *cq) {
 	
 	if (srv_socket->is_ssl) {
 #ifdef USE_OPENSSL
-		ret = network_write_chunkqueue_openssl(srv, con, con->ssl, cq);
+		ret = srv->network_ssl_backend_write(srv, con, con->ssl, cq);
 #endif
 	} else {
-		/* dispatch call */
-#if defined USE_LINUX_SENDFILE
-		ret = network_write_chunkqueue_linuxsendfile(srv, con, con->fd, cq); 
-#elif defined USE_FREEBSD_SENDFILE
-		ret = network_write_chunkqueue_freebsdsendfile(srv, con, con->fd, cq); 
-#elif defined USE_SOLARIS_SENDFILEV
-		ret = network_write_chunkqueue_solarissendfilev(srv, con, con->fd, cq); 
-#elif defined USE_WRITEV
-		ret = network_write_chunkqueue_writev(srv, con, con->fd, cq);
-#else
-		ret = network_write_chunkqueue_write(srv, con, con->fd, cq);
-#endif
+		ret = srv->network_backend_write(srv, con, con->fd, cq);
 	}
 	
 	if (ret >= 0) {
