@@ -337,8 +337,8 @@ int http_request_parse(server *srv, connection *con) {
 					con->response.keep_alive = 0;
 					con->keep_alive = 0;
 					
-					log_error_write(srv, __FILE__, __LINE__, "s", "incomplete request line -> 400");
 					if (srv->srvconf.log_request_header_on_error) {
+						log_error_write(srv, __FILE__, __LINE__, "s", "incomplete request line -> 400");
 						log_error_write(srv, __FILE__, __LINE__, "Sb",
 								"request-header:\n",
 								con->request.request);
@@ -357,13 +357,13 @@ int http_request_parse(server *srv, connection *con) {
 					con->response.keep_alive = 0;
 					con->keep_alive = 0;
 					
-					log_error_write(srv, __FILE__, __LINE__, "s", "unknown http-method -> 501");
 					if (srv->srvconf.log_request_header_on_error) {
-							log_error_write(srv, __FILE__, __LINE__, "Sb",
-									"request-header:\n",
-									con->request.request);
-						}
-					
+						log_error_write(srv, __FILE__, __LINE__, "s", "unknown http-method -> 501");
+						log_error_write(srv, __FILE__, __LINE__, "Sb",
+								"request-header:\n",
+								con->request.request);
+					}
+				
 					return 0;
 				}
 				
@@ -377,8 +377,8 @@ int http_request_parse(server *srv, connection *con) {
 					} else { 
 						con->http_status = 505;
 
-						log_error_write(srv, __FILE__, __LINE__, "s", "unknown HTTP version -> 505");
 						if (srv->srvconf.log_request_header_on_error) {
+							log_error_write(srv, __FILE__, __LINE__, "s", "unknown HTTP version -> 505");
 							log_error_write(srv, __FILE__, __LINE__, "Sb",
 									"request-header:\n",
 									con->request.request);
@@ -389,12 +389,12 @@ int http_request_parse(server *srv, connection *con) {
 					con->http_status = 400;
 					con->keep_alive = 0;
 
-					log_error_write(srv, __FILE__, __LINE__, "s", "unknown protocol -> 400");
 					if (srv->srvconf.log_request_header_on_error) {
-							log_error_write(srv, __FILE__, __LINE__, "Sb",
-									"request-header:\n",
-									con->request.request);
-						}
+						log_error_write(srv, __FILE__, __LINE__, "s", "unknown protocol -> 400");
+						log_error_write(srv, __FILE__, __LINE__, "Sb",
+								"request-header:\n",
+								con->request.request);
+					}
 					return 0;
 				}
 				
@@ -415,23 +415,23 @@ int http_request_parse(server *srv, connection *con) {
 						con->http_status = 400;
 						con->keep_alive = 0;
 						
-						buf[0] = con->request.uri->ptr[j];
-						buf[1] = '\0';
-					
-						if (con->request.uri->ptr[j] > 32 &&
-						    con->request.uri->ptr[j] != 127) {	
-							/* the character is printable -> print it */
-							log_error_write(srv, __FILE__, __LINE__, "ss",
-									"invalid character in URI -> 400",
-									buf);
-						} else {
-							/* a control-character, print ascii-code */
-							log_error_write(srv, __FILE__, __LINE__, "sd",
-									"invalid character in URI -> 400",
-									con->request.uri->ptr[j]);
-						}
-						
 						if (srv->srvconf.log_request_header_on_error) {
+							buf[0] = con->request.uri->ptr[j];
+							buf[1] = '\0';
+					
+							if (con->request.uri->ptr[j] > 32 &&
+							    con->request.uri->ptr[j] != 127) {	
+								/* the character is printable -> print it */
+								log_error_write(srv, __FILE__, __LINE__, "ss",
+										"invalid character in URI -> 400",
+										buf);
+							} else {
+								/* a control-character, print ascii-code */
+								log_error_write(srv, __FILE__, __LINE__, "sd",
+										"invalid character in URI -> 400",
+										con->request.uri->ptr[j]);
+							}
+						
 							log_error_write(srv, __FILE__, __LINE__, "Sb",
 									"request-header:\n",
 									con->request.request);
@@ -468,8 +468,8 @@ int http_request_parse(server *srv, connection *con) {
 				con->response.keep_alive = 0;
 				con->keep_alive = 0;
 				
-				log_error_write(srv, __FILE__, __LINE__, "s", "overlong request line -> 400");
 				if (srv->srvconf.log_request_header_on_error) {
+					log_error_write(srv, __FILE__, __LINE__, "s", "overlong request line -> 400");
 					log_error_write(srv, __FILE__, __LINE__, "Sb",
 							"request-header:\n",
 							con->request.request);
@@ -576,7 +576,12 @@ int http_request_parse(server *srv, connection *con) {
 					default:
 						/* error */
 						
-						log_error_write(srv, __FILE__, __LINE__, "s", "WS character in key -> 400");
+						if (srv->srvconf.log_request_header_on_error) {
+							log_error_write(srv, __FILE__, __LINE__, "s", "WS character in key -> 400");
+							log_error_write(srv, __FILE__, __LINE__, "Sb",
+								"request-header:\n",
+								con->request.request);
+						}
 					
 						con->http_status = 400;
 						con->response.keep_alive = 0;
@@ -599,7 +604,12 @@ int http_request_parse(server *srv, connection *con) {
 					
 					break;
 				} else {
-					log_error_write(srv, __FILE__, __LINE__, "s", "CR without LF -> 400");
+					if (srv->srvconf.log_request_header_on_error) {
+						log_error_write(srv, __FILE__, __LINE__, "s", "CR without LF -> 400");
+						log_error_write(srv, __FILE__, __LINE__, "Sb",
+							"request-header:\n",
+							con->request.request);
+					}
 					
 					con->http_status = 400;
 					con->keep_alive = 0;
@@ -642,8 +652,14 @@ int http_request_parse(server *srv, connection *con) {
 				con->keep_alive = 0;
 				con->response.keep_alive = 0;
 				
-				log_error_write(srv, __FILE__, __LINE__, "sbsds", 
+				if (srv->srvconf.log_request_header_on_error) {
+					log_error_write(srv, __FILE__, __LINE__, "sbsds", 
 						"CTL character in key", con->request.request, cur, *cur, "-> 400");
+
+					log_error_write(srv, __FILE__, __LINE__, "Sb",
+						"request-header:\n",
+						con->request.request);
+				}
 				
 				return 0;
 			default:
@@ -662,7 +678,14 @@ int http_request_parse(server *srv, connection *con) {
 						if (!ds) {
 							/* 400 */
 					
-							log_error_write(srv, __FILE__, __LINE__, "s", "WS at the start of first line -> 400");
+							if (srv->srvconf.log_request_header_on_error) {
+								log_error_write(srv, __FILE__, __LINE__, "s", "WS at the start of first line -> 400");
+							
+								log_error_write(srv, __FILE__, __LINE__, "Sb",
+									"request-header:\n",
+									con->request.request);
+							}
+
 					
 							con->http_status = 400;
 							con->keep_alive = 0;
@@ -725,9 +748,9 @@ int http_request_parse(server *srv, connection *con) {
 									con->http_status = 400;
 									con->keep_alive = 0;
 									
-									log_error_write(srv, __FILE__, __LINE__, "s", 
-											"duplicate Content-Length-header -> 400");
 									if (srv->srvconf.log_request_header_on_error) {
+										log_error_write(srv, __FILE__, __LINE__, "s", 
+												"duplicate Content-Length-header -> 400");
 										log_error_write(srv, __FILE__, __LINE__, "Sb",
 												"request-header:\n",
 												con->request.request);
@@ -774,9 +797,9 @@ int http_request_parse(server *srv, connection *con) {
 									con->http_status = 400;
 									con->keep_alive = 0;
 									
-									log_error_write(srv, __FILE__, __LINE__, "s", 
-											"duplicate Content-Type-header -> 400");
 									if (srv->srvconf.log_request_header_on_error) {
+										log_error_write(srv, __FILE__, __LINE__, "s", 
+												"duplicate Content-Type-header -> 400");
 										log_error_write(srv, __FILE__, __LINE__, "Sb",
 												"request-header:\n",
 												con->request.request);
@@ -791,6 +814,12 @@ int http_request_parse(server *srv, connection *con) {
 								 *   -> (10.4.18) 417 (close)
 								 * 
 								 * (not handled at all yet, we always send 417 here)
+								 *
+								 * What has to be added ?
+								 * 1. handling of chunked request body
+								 * 2. out-of-order sending from the HTTP/1.1 100 Continue
+								 *    header
+								 *
 								 */
 								
 								con->http_status = 417;
@@ -805,9 +834,9 @@ int http_request_parse(server *srv, connection *con) {
 									con->http_status = 400;
 									con->keep_alive = 0;
 									
-									log_error_write(srv, __FILE__, __LINE__, "s", 
-											"duplicate Host-header -> 400");
 									if (srv->srvconf.log_request_header_on_error) {
+										log_error_write(srv, __FILE__, __LINE__, "s", 
+												"duplicate Host-header -> 400");
 										log_error_write(srv, __FILE__, __LINE__, "Sb",
 												"request-header:\n",
 												con->request.request);
@@ -815,16 +844,21 @@ int http_request_parse(server *srv, connection *con) {
 									return 0;
 								}
 							} else if (cmp > 0 && 0 == (cmp = buffer_caseless_compare(CONST_BUF_LEN(ds->key), CONST_STR_LEN("If-Modified-Since")))) {
-								/* if dup, only the first one will survive */
+								/* Proxies sometimes send dup headers
+								 * if they are the same we ignore the second
+								 * if not, we raise an error */
 								if (!con->request.http_if_modified_since) {
 									con->request.http_if_modified_since = ds->value->ptr;
+								} else if (0 == strcasecmp(con->request.http_if_modified_since,
+											ds->value->ptr)) {
+									/* ignore it if they are the same */
 								} else {
 									con->http_status = 400;
 									con->keep_alive = 0;
 									
-									log_error_write(srv, __FILE__, __LINE__, "s", 
-											"duplicate If-Modified-Since header -> 400");
 									if (srv->srvconf.log_request_header_on_error) {
+										log_error_write(srv, __FILE__, __LINE__, "s", 
+												"duplicate If-Modified-Since header -> 400");
 										log_error_write(srv, __FILE__, __LINE__, "Sb",
 												"request-header:\n",
 												con->request.request);
@@ -839,9 +873,9 @@ int http_request_parse(server *srv, connection *con) {
 									con->http_status = 400;
 									con->keep_alive = 0;
 									
-									log_error_write(srv, __FILE__, __LINE__, "s", 
-											"duplicate If-None-Match-header -> 400");
 									if (srv->srvconf.log_request_header_on_error) {
+										log_error_write(srv, __FILE__, __LINE__, "s", 
+												"duplicate If-None-Match-header -> 400");
 										log_error_write(srv, __FILE__, __LINE__, "Sb",
 												"request-header:\n",
 												con->request.request);
@@ -862,9 +896,9 @@ int http_request_parse(server *srv, connection *con) {
 									con->http_status = 400;
 									con->keep_alive = 0;
 									
-									log_error_write(srv, __FILE__, __LINE__, "s", 
-											"duplicate Host-header -> 400");
 									if (srv->srvconf.log_request_header_on_error) {
+										log_error_write(srv, __FILE__, __LINE__, "s", 
+												"duplicate Range-header -> 400");
 										log_error_write(srv, __FILE__, __LINE__, "Sb",
 												"request-header:\n",
 												con->request.request);
@@ -886,8 +920,10 @@ int http_request_parse(server *srv, connection *con) {
 					key_len = 0;
 					in_folding = 0;
 				} else {
-					log_error_write(srv, __FILE__, __LINE__, "sbs", 
-							"CR without LF", con->request.request, "-> 400");
+					if (srv->srvconf.log_request_header_on_error) {
+						log_error_write(srv, __FILE__, __LINE__, "sbs", 
+								"CR without LF", con->request.request, "-> 400");
+					}
 					
 					con->http_status = 400;
 					con->keep_alive = 0;
@@ -926,8 +962,8 @@ int http_request_parse(server *srv, connection *con) {
 			con->response.keep_alive = 0;
 			con->keep_alive = 0;
 			
-			log_error_write(srv, __FILE__, __LINE__, "s", "HTTP/1.1 but Host missing -> 400");
 			if (srv->srvconf.log_request_header_on_error) {
+				log_error_write(srv, __FILE__, __LINE__, "s", "HTTP/1.1 but Host missing -> 400");
 				log_error_write(srv, __FILE__, __LINE__, "Sb",
 						"request-header:\n",
 						con->request.request);
@@ -948,10 +984,10 @@ int http_request_parse(server *srv, connection *con) {
 	/* check hostname field if it is set */
 	if (NULL != con->request.http_host &&
 	    0 != request_check_hostname(srv, con, con->request.http_host)) {
-		log_error_write(srv, __FILE__, __LINE__, "s",
-				"Invalid Hostname -> 400");
 		
 		if (srv->srvconf.log_request_header_on_error) {
+			log_error_write(srv, __FILE__, __LINE__, "s",
+					"Invalid Hostname -> 400");
 			log_error_write(srv, __FILE__, __LINE__, "Sb",
 					"request-header:\n",
 					con->request.request);
@@ -963,33 +999,43 @@ int http_request_parse(server *srv, connection *con) {
 		
 		return 0;
 	}
-	
-	/* check if we have read post data */
-	if (con->request.http_method == HTTP_METHOD_POST
-         || (con->request.http_method != HTTP_METHOD_GET
-            && con->request.http_method != HTTP_METHOD_HEAD
-            && con->request.http_method != HTTP_METHOD_OPTIONS
-            && con_length_set))
-   {
 
-		server_socket *srv_socket = con->srv_socket;
-#if 0	
-		if (con->request.http_content_type == NULL) {
+	switch(con->request.http_method) {
+	case HTTP_METHOD_GET:
+	case HTTP_METHOD_HEAD:
+	case HTTP_METHOD_OPTIONS:
+		/* content-length is forbidden for those */
+		if (con_length_set && con->request.content_length != 0) {
+			/* content-length is missing */
 			log_error_write(srv, __FILE__, __LINE__, "s", 
-					"Content-Length request, but content-type not set");
+					"GET/HEAD/OPTIONS with content-length -> 400");
+
+			con->keep_alive = 0;
+			con->http_status = 400;
+			return 0;
 		}
-#endif
-		
-		if (con_length_set == 0) {
+		break;
+	case HTTP_METHOD_POST:
+		/* content-length is required for them */
+		if (!con_length_set) {
 			/* content-length is missing */
 			log_error_write(srv, __FILE__, __LINE__, "s", 
 					"POST-request, but content-length missing -> 411");
+
 			con->keep_alive = 0;
-			
 			con->http_status = 411;
 			return 0;
+
 		}
-		
+		break;
+	default:
+		/* the may have a content-length */
+		break;
+	}
+			
+	
+	/* check if we have read post data */
+	if (con_length_set) {
 		/* don't handle more the SSIZE_MAX bytes in content-length */
 		if (con->request.content_length > SSIZE_MAX) {
 			con->http_status = 413; 
