@@ -28,6 +28,7 @@
 #include "stat_cache.h"
 #include "plugin.h"
 #include "joblist.h"
+#include "network_backends.h"
 
 #ifdef HAVE_GETOPT_H
 #include <getopt.h>
@@ -50,13 +51,17 @@
 #include <sys/resource.h>
 #endif
 
+#ifdef HAVE_SYS_PRCTL_H
+#include <sys/prctl.h>
+#endif
+
 #ifndef __sgi
 /* IRIX doesn't like the alarm based time() optimization */
 /* #define USE_ALARM */
 #endif
 
 static volatile sig_atomic_t srv_shutdown = 0;
-static volatile sig_atomic_t gracefull_shutdown = 0;
+static volatile sig_atomic_t graceful_shutdown = 0;
 static volatile sig_atomic_t handle_sig_alarm = 1;
 static volatile sig_atomic_t handle_sig_hup = 0;
 
@@ -67,7 +72,11 @@ static void sigaction_handler(int sig, siginfo_t *si, void *context) {
 
 	switch (sig) {
 	case SIGTERM: srv_shutdown = 1; break;
-	case SIGINT: gracefull_shutdown = 1; break;
+	case SIGINT: 
+	     if (graceful_shutdown) srv_shutdown = 1;
+	     else graceful_shutdown = 1; 
+
+	     break;
 	case SIGALRM: handle_sig_alarm = 1; break;
 	case SIGHUP:  handle_sig_hup = 1; break;
 	case SIGCHLD: break;
@@ -77,7 +86,11 @@ static void sigaction_handler(int sig, siginfo_t *si, void *context) {
 static void signal_handler(int sig) {
 	switch (sig) {
 	case SIGTERM: srv_shutdown = 1; break;
-	case SIGINT: gracefull_shutdown = 1; break;
+	case SIGINT: 
+	     if (graceful_shutdown) srv_shutdown = 1;
+	     else graceful_shutdown = 1; 
+
+	     break;
 	case SIGALRM: handle_sig_alarm = 1; break;
 	case SIGHUP:  handle_sig_hup = 1; break;
 	case SIGCHLD:  break;
@@ -96,15 +109,15 @@ static void daemonize(void) {
 #ifdef SIGTSTP
 	signal(SIGTSTP, SIG_IGN);
 #endif
-	if (fork() != 0) exit(0);
+	if (0 != fork()) exit(0);
 	
-	if (setsid() == -1) exit(0);
+	if (-1 == setsid()) exit(0);
 
 	signal(SIGHUP, SIG_IGN);
 
-	if (fork() != 0) exit(0);
+	if (0 != fork()) exit(0);
 	
-	chdir("/");
+	if (0 != chdir("/")) exit(0);
 	
 	umask(0);
 }
@@ -144,6 +157,7 @@ static server *server_init(void) {
 	
 	CLEAN(config_context);
 	CLEAN(config_touched);
+	CLEAN(status);
 #undef CLEAN
 	
 	for (i = 0; i < FILE_CACHE_MAX; i++) {
@@ -164,6 +178,8 @@ static server *server_init(void) {
 	
 	srv->srvconf.modules = array_init();
 	srv->srvconf.modules_dir = buffer_init_string(LIBRARY_DIR);
+	srv->srvconf.network_backend = buffer_init();
+	srv->srvconf.upload_tempdirs = array_init();
 	
 	/* use syslog */
 	srv->errorlog_fd = -1;
@@ -239,6 +255,7 @@ static void server_free(server *srv) {
 	
 	CLEAN(config_context);
 	CLEAN(config_touched);
+	CLEAN(status);
 #undef CLEAN
 	
 	joblist_free(srv, srv->joblist);
@@ -268,6 +285,135 @@ static void show_version (void) {
 	write(STDOUT_FILENO, b, strlen(b));
 }
 
+static void show_features (void) {
+  show_version();
+  printf("\nEvent Handlers:\n\n%s",
+
+#ifdef USE_SELECT
+      "\t+ select (generic)\n"
+#else
+      "\t- select (generic)\n"
+#endif
+#ifdef USE_POLL
+      "\t+ poll (Unix)\n"
+#else
+      "\t- poll (Unix)\n"
+#endif
+#ifdef USE_LINUX_SIGIO
+      "\t+ rt-signals (Linux 2.4+)\n"
+#else
+      "\t- rt-signals (Linux 2.4+)\n"
+#endif
+#ifdef USE_LINUX_EPOLL
+      "\t+ epoll (Linux 2.6)\n"
+#else
+      "\t- epoll (Linux 2.6)\n"
+#endif
+#ifdef USE_SOLARIS_DEVPOLL
+      "\t+ /dev/poll (Solaris)\n"
+#else
+      "\t- /dev/poll (Solaris)\n"
+#endif
+#ifdef USE_FREEBSD_KQUEUE
+      "\t+ kqueue (FreeBSD)\n"
+#else
+      "\t- kqueue (FreeBSD)\n"
+#endif
+      "\nNetwork handler:\n\n"
+#if defined(USE_LINUX_SENDFILE) || defined(USE_FREEBSD_SENDFILE) || defined(USE_SOLARIS_SENDFILEV) || defined(USE_AIX_SENDFILE)
+      "\t+ sendfile\n"
+#else
+  #ifdef USE_WRITEV
+      "\t+ writev\n"
+  #else
+      "\t+ write\n"
+  #endif
+  #ifdef USE_MMAP
+      "\t+ mmap support\n"
+  #else
+      "\t- mmap support\n"
+  #endif
+#endif
+      "\nFeatures:\n\n"
+#ifdef HAVE_IPV6
+      "\t+ IPv6 support\n"
+#else
+      "\t- IPv6 support\n"
+#endif
+#if defined HAVE_ZLIB_H && defined HAVE_LIBZ
+      "\t+ zlib support\n"
+#else
+      "\t- zlib support\n"
+#endif
+#if defined HAVE_BZLIB_H && defined HAVE_LIBBZ2
+      "\t+ bzip2 support\n"
+#else
+      "\t- bzip2 support\n"
+#endif
+#ifdef HAVE_LIBCRYPT
+      "\t+ crypt support\n"
+#else
+      "\t- crypt support\n"
+#endif
+#ifdef USE_PAM
+      "\t+ PAM support\n"
+#else
+      "\t- PAM support\n"
+#endif
+#ifdef USE_OPENSSL
+      "\t+ SSL Support\n"
+#else
+      "\t- SSL Support\n"
+#endif
+#ifdef HAVE_LIBPCRE
+      "\t+ PCRE support\n"
+#else
+      "\t- PCRE support\n"
+#endif
+#ifdef HAVE_MYSQL
+      "\t+ mySQL support\n"
+#else
+      "\t- mySQL support\n"
+#endif
+#if defined(HAVE_LDAP_H) && defined(HAVE_LBER_H) && defined(HAVE_LIBLDAP) && defined(HAVE_LIBLBER)
+      "\t+ LDAP support\n"
+#else
+      "\t- LDAP support\n"
+#endif
+#ifdef HAVE_MEMCACHE_H
+      "\t+ memcached support\n"
+#else
+      "\t- memcached support\n"
+#endif
+#ifdef HAVE_FAM_H
+      "\t+ FAM support\n"
+#else
+      "\t- FAM support\n"
+#endif
+#ifdef HAVE_LUA_H
+      "\t+ LUA support\n"
+#else
+      "\t- LUA support\n"
+#endif
+#ifdef HAVE_LIBXML_H
+      "\t+ xml support\n"
+#else
+      "\t- xml support\n"
+#endif
+#ifdef HAVE_SQLITE3_H
+      "\t+ SQLite support\n"
+#else
+      "\t- SQLite support\n"
+#endif
+#ifdef HAVE_GDBM_H
+      "\t+ GDBM support\n"
+#else
+      "\t- GDBM support\n"
+#endif
+      "\n"
+      );
+}
+
 static void show_help (void) {
 #ifdef USE_OPENSSL
 # define TEXT_SSL " (ssl)"
@@ -283,6 +429,7 @@ static void show_help (void) {
 " -t         test the config-file, and exit\n" \
 " -D         don't go to background (default: go to background)\n" \
 " -v         show version\n" \
+" -V         show compile-time features\n" \
 " -h         show this help\n" \
 "\n"
 ;
@@ -321,7 +468,7 @@ int main (int argc, char **argv) {
 	setlocale(LC_TIME, "C");
 	
 	if (NULL == (srv = server_init())) {
-		fprintf(stderr, "did this really happend ?\n");
+		fprintf(stderr, "did this really happen?\n");
 		return -1;
 	}
 	
@@ -335,7 +482,7 @@ int main (int argc, char **argv) {
 #endif
 	srv->srvconf.dont_daemonize = 0;
 	
-	while(-1 != (o = getopt(argc, argv, "f:m:hvDpt"))) {
+	while(-1 != (o = getopt(argc, argv, "f:m:hvVDpt"))) {
 		switch(o) {
 		case 'f': 
 			if (config_read(srv, optarg)) { 
@@ -350,6 +497,7 @@ int main (int argc, char **argv) {
 		case 't': test_config = 1; break;
 		case 'D': srv->srvconf.dont_daemonize = 1; break;
 		case 'v': show_version(); return 0;
+		case 'V': show_features(); return 0;          
 		case 'h': show_help(); return 0;
 		default: 
 			show_help();
@@ -511,11 +659,20 @@ int main (int argc, char **argv) {
 			}
 		}
 
+		/* #372: solaris need some fds extra for devpoll */	
+		if (rlim.rlim_cur > 10) rlim.rlim_cur -= 10;
+
 		if (srv->event_handler == FDEVENT_HANDLER_SELECT) {
 			srv->max_fds = rlim.rlim_cur < FD_SETSIZE - 200 ? rlim.rlim_cur : FD_SETSIZE - 200;
 		} else {
 			srv->max_fds = rlim.rlim_cur;
 		}
+
+		/* set core file rlimit, if enable_cores is set */
+		if (use_rlimit && srv->srvconf.enable_cores && getrlimit(RLIMIT_CORE, &rlim) == 0) {
+			rlim.rlim_cur = rlim.rlim_max;
+			setrlimit(RLIMIT_CORE, &rlim);
+		}
 #endif
 		if (srv->event_handler == FDEVENT_HANDLER_SELECT) {
 			/* don't raise the limit above FD_SET_SIZE */
@@ -527,11 +684,6 @@ int main (int argc, char **argv) {
 			}
 		}
 
-		if (NULL == (srv->ev = fdevent_init(srv->max_fds + 1, srv->event_handler))) {
-			log_error_write(srv, __FILE__, __LINE__,
-					"s", "fdevent_init failed");
-			return -1;
-		}
 		
 #ifdef HAVE_PWD_H
 		/* set user and group */
@@ -592,6 +744,11 @@ int main (int argc, char **argv) {
 		if (srv->srvconf.username->used && srv->srvconf.groupname->used)
 			initgroups(srv->srvconf.username->ptr, grp->gr_gid);
 		if (srv->srvconf.username->used) setuid(pwd->pw_uid);
+#endif
+#ifdef HAVE_PRCTL
+		if (srv->srvconf.enable_cores) {
+			prctl(PR_SET_DUMPABLE, 1, 0, 0, 0);
+		}
 #endif
 	} else {
 
@@ -608,6 +765,13 @@ int main (int argc, char **argv) {
 		} else {
 			srv->max_fds = rlim.rlim_cur;
 		}
+
+		/* set core file rlimit, if enable_cores is set */
+		if (srv->srvconf.enable_cores && getrlimit(RLIMIT_CORE, &rlim) == 0) {
+			rlim.rlim_cur = rlim.rlim_max;
+			setrlimit(RLIMIT_CORE, &rlim);
+		}
+
 #endif
 		if (srv->event_handler == FDEVENT_HANDLER_SELECT) {
 			/* don't raise the limit above FD_SET_SIZE */
@@ -618,13 +782,6 @@ int main (int argc, char **argv) {
 				return -1;
 			}
 		}
-
-	
-		if (NULL == (srv->ev = fdevent_init(srv->max_fds + 1, srv->event_handler))) {
-			log_error_write(srv, __FILE__, __LINE__,
-					"s", "fdevent_init failed");
-			return -1;
-		}
 		
 		if (0 != network_init(srv)) {
 			plugins_free(srv);
@@ -639,7 +796,7 @@ int main (int argc, char **argv) {
 		/* we can't have more connections than max-fds */
 		srv->max_conns = srv->max_fds;
 	} else if (srv->srvconf.max_conns) {
-		/* otherwise respect thw wishes of the user */
+		/* otherwise respect the wishes of the user */
 		srv->max_conns = srv->srvconf.max_conns;
 	} else {
 		/* or use the default */
@@ -726,22 +883,6 @@ int main (int argc, char **argv) {
 		return -1;
 	}
 	
-	/* 
-	 * kqueue() is called here, select resets its internals, 
-	 * all server sockets get their handlers
-	 *
-	 * */
-	if (0 != network_register_fdevents(srv)) {
-		plugins_free(srv);
-		network_close(srv);
-		server_free(srv);
-		
-		return -1;
-	}
-	
-	/* get the current number of FDs */
-	srv->cur_fds = open("/dev/null", O_RDONLY);
-	close(srv->cur_fds);
 	
 #ifdef HAVE_SIGACTION
 	memset(&act, 0, sizeof(act));
@@ -786,24 +927,6 @@ int main (int argc, char **argv) {
 	getitimer(ITIMER_REAL, &interval);
 #endif
 
-	/* might fail if user is using fam (not gamin) and famd isn't running */
-	if (NULL == (srv->stat_cache = stat_cache_init())) {
-#if defined(HAVE_FAM_H) && !defined(HAVE_FAMNOEXISTS)
-		log_error_write(srv, __FILE__, __LINE__, "s",
-			"FAMOpen2() failed.  Is famd running?");
-		return -1;
-#else
-		SEGFAULT();
-#endif
-	}
-
-#ifdef HAVE_FAM_H
-	/* setup FAM */
-	srv->stat_cache->fam_fcce_ndx = -1;
-	fdevent_register(srv->ev, FAMCONNECTION_GETFD(srv->stat_cache->fam), stat_cache_handle_fdevent, NULL);
-	fdevent_event_add(srv->ev, &(srv->stat_cache->fam_fcce_ndx), FAMCONNECTION_GETFD(srv->stat_cache->fam), FDEVENT_IN);
-#endif
-
 #ifdef HAVE_FORK	
 	/* start watcher and workers */
 	num_childs = srv->srvconf.max_worker;
@@ -827,10 +950,69 @@ int main (int argc, char **argv) {
 				num_childs++;
 			}
 		}
+		if (srv_shutdown) {
+			kill(0, SIGTERM);
+		}
 		if (!child) return 0;
 	}
 #endif
 
+	if (NULL == (srv->ev = fdevent_init(srv->max_fds + 1, srv->event_handler))) {
+		log_error_write(srv, __FILE__, __LINE__,
+				"s", "fdevent_init failed");
+		return -1;
+	}
+	/* 
+	 * kqueue() is called here, select resets its internals, 
+	 * all server sockets get their handlers
+	 *
+	 * */
+	if (0 != network_register_fdevents(srv)) {
+		plugins_free(srv);
+		network_close(srv);
+		server_free(srv);
+		
+		return -1;
+	}
+
+	/* might fail if user is using fam (not gamin) and famd isn't running */
+	if (NULL == (srv->stat_cache = stat_cache_init())) {
+		log_error_write(srv, __FILE__, __LINE__, "s",
+			"stat-cache could not be setup, dieing.");
+		return -1;
+	}
+
+#ifdef HAVE_FAM_H
+	/* setup FAM */
+	if (srv->srvconf.stat_cache_engine == STAT_CACHE_ENGINE_FAM) {
+		if (0 != FAMOpen2(srv->stat_cache->fam, "lighttpd")) {
+			log_error_write(srv, __FILE__, __LINE__, "s", 
+					 "could not open a fam connection, dieing.");
+			return -1;
+		}
+#ifdef HAVE_FAMNOEXISTS
+		FAMNoExists(srv->stat_cache->fam);
+#endif
+
+		srv->stat_cache->fam_fcce_ndx = -1;
+		fdevent_register(srv->ev, FAMCONNECTION_GETFD(srv->stat_cache->fam), stat_cache_handle_fdevent, NULL);
+		fdevent_event_add(srv->ev, &(srv->stat_cache->fam_fcce_ndx), FAMCONNECTION_GETFD(srv->stat_cache->fam), FDEVENT_IN);
+	}
+#endif
+
+
+	/* get the current number of FDs */
+	srv->cur_fds = open("/dev/null", O_RDONLY);
+	close(srv->cur_fds);
+
+	for (i = 0; i < srv->srv_sockets.used; i++) {
+		server_socket *srv_socket = srv->srv_sockets.ptr[i];
+		if (-1 == fdevent_fcntl_set(srv->ev, srv_socket->fd)) {
+			log_error_write(srv, __FILE__, __LINE__, "ss", "fcntl failed:", strerror(errno));
+			return -1;
+		}
+	}
+
 	/* main-loop */
 	while (!srv_shutdown) {
 		int n;
@@ -941,8 +1123,14 @@ int main (int argc, char **argv) {
 						if (srv->cur_ts - con->write_request_ts > con->conf.max_write_idle) {
 							/* time - out */
 #if 1
-							log_error_write(srv, __FILE__, __LINE__, "sd", 
-									"connection closed - write-request-timeout:", con->fd);
+							log_error_write(srv, __FILE__, __LINE__, "sbsosds", 
+									"NOTE: a request for",
+									con->request.uri,
+									"timed out after writing",
+									con->bytes_written,
+									"bytes. We waited",
+									(int)con->conf.max_write_idle,
+									"seconds. If this a problem increase server.max-write-idle");
 #endif
 							connection_set_state(srv, con, CON_STATE_ERROR);
 							changed = 1;
@@ -952,7 +1140,8 @@ int main (int argc, char **argv) {
 					if (0 == (t_diff = srv->cur_ts - con->connection_start)) t_diff = 1;
 	
 					if (con->traffic_limit_reached && 
-					    ((con->bytes_written / t_diff) < con->conf.kbytes_per_second * 1024)) {
+					    (con->conf.kbytes_per_second == 0 || 
+					     ((con->bytes_written / t_diff) < con->conf.kbytes_per_second * 1024))) {
 						/* enable connection again */
 						con->traffic_limit_reached = 0;
 						
@@ -987,7 +1176,7 @@ int main (int argc, char **argv) {
 
 			if ((srv->cur_fds + srv->want_fds < srv->max_fds * 0.8) && /* we have enough unused fds */
 			    (srv->conns->used < srv->max_conns * 0.9) &&
-			    (0 == gracefull_shutdown)) {
+			    (0 == graceful_shutdown)) {
 				for (i = 0; i < srv->srv_sockets.used; i++) {
 					server_socket *srv_socket = srv->srv_sockets.ptr[i];
 					fdevent_event_add(srv->ev, &(srv_socket->fde_ndx), srv_socket->fd, FDEVENT_IN);
@@ -1000,7 +1189,7 @@ int main (int argc, char **argv) {
 		} else {
 			if ((srv->cur_fds + srv->want_fds > srv->max_fds * 0.9) || /* out of fds */
 			    (srv->conns->used > srv->max_conns) || /* out of connections */
-			    (gracefull_shutdown)) { /* gracefull_shutdown */ 
+			    (graceful_shutdown)) { /* graceful_shutdown */ 
 
 				/* disable server-fds */
 			
@@ -1008,11 +1197,11 @@ int main (int argc, char **argv) {
 					server_socket *srv_socket = srv->srv_sockets.ptr[i];
 					fdevent_event_del(srv->ev, &(srv_socket->fde_ndx), srv_socket->fd);
 
-					if (gracefull_shutdown) {
+					if (graceful_shutdown) {
 						/* we don't want this socket anymore,
 						 *
 						 * closing it right away will make it possible for
-						 * the next lighttpd to take over (gracefull restart)
+						 * the next lighttpd to take over (graceful restart)
 						 *  */
 
 						fdevent_unregister(srv->ev, srv_socket->fd);
@@ -1023,8 +1212,8 @@ int main (int argc, char **argv) {
 					}
 				}
 		
-				if (gracefull_shutdown) {
-					log_error_write(srv, __FILE__, __LINE__, "s", "[note] gracefull shutdown started");
+				if (graceful_shutdown) {
+					log_error_write(srv, __FILE__, __LINE__, "s", "[note] graceful shutdown started");
 				} else if (srv->conns->used > srv->max_conns) {
 					log_error_write(srv, __FILE__, __LINE__, "s", "[note] sockets disabled, connection limit reached");
 				} else {
@@ -1035,8 +1224,8 @@ int main (int argc, char **argv) {
 			}
 		}
 
-		if (gracefull_shutdown && srv->conns->used == 0) {
-			/* we are in gracefull shutdown phase and all connections are closed
+		if (graceful_shutdown && srv->conns->used == 0) {
+			/* we are in graceful shutdown phase and all connections are closed
 			 * we are ready to terminate without harming anyone */
 			srv_shutdown = 1;
 		}
